package dk.aau.cs.model.tapn.Colored.Expressions;

import dk.aau.cs.TCTL.StringPosition;
import dk.aau.cs.TCTL.TCTLAbstractStateProperty;
import dk.aau.cs.model.tapn.Colored.Color;
import dk.aau.cs.model.tapn.Colored.ColorType;
import dk.aau.cs.model.tapn.Colored.ExpressionSupport.ExprStringPosition;
import dk.aau.cs.model.tapn.Colored.ExpressionSupport.ExprValues;
import dk.aau.cs.model.tapn.Colored.ProductType;
import dk.aau.cs.model.tapn.Colored.Variable;

import java.util.Set;
import java.util.Vector;

public class TupleExpression extends ColorExpression {
    private Vector<ColorExpression> colors;

    public Vector<ColorExpression> getColors() {
        return colors;
    }

    public TupleExpression(Vector<ColorExpression> colors) {
        this.colors = colors;
    }

    public Color eval(ExpressionContext context) {
        Vector<Color> colors = new Vector<Color>();
        Vector<ColorType> colorTypes = new Vector<ColorType>();

        for (ColorExpression ce : this.colors) {
            Color color = ce.eval(context);
            colors.add(color);
            colorTypes.add(color.getColorType());
        }

        ProductType pt = context.findProductColorType(colorTypes);
        return pt.getColor(colors);
    }

    public void addColorExpression(ColorExpression expr) {
        colors.add(expr);
    }

    @Override
    public boolean isSimpleProperty() {return false;}


    @Override
    public ColorExpression replace(Expression object1, Expression object2) {
        if (object1 == this && object2 instanceof ColorExpression) {
            ColorExpression obj2 = (ColorExpression) object2;
            obj2.setParent(parent);
            return obj2;
        }
        else  {
            for (int i = 0; i < colors.size(); i++) {
                colors.set(i, colors.get(i).replace(object1, object2));
            }
            return this;
        }
    }

    @Override
    public Expression copy() {
        return new TupleExpression(colors);
    }

    @Override
    public boolean containsPlaceHolder() {
        for (ColorExpression expr : colors) {
            if (expr.containsPlaceHolder()) {
                return true;
            }
        }
        return false;
    }

    @Override
    public ColorExpression findFirstPlaceHolder() {
        for (ColorExpression expr : colors) {
            if (expr.containsPlaceHolder()) {
                return expr.findFirstPlaceHolder();
            }
        }
        return null;
    }

    @Override
    public ExprValues getValues(ExprValues exprValues) {
        for (ColorExpression color : colors) {
            exprValues = color.getValues(exprValues);
        }
        return exprValues;
    }

    public void getVariables(Set<Variable> variables) {
        for (ColorExpression element : colors) {
            element.getVariables(variables);
        }
    }

    public String toString() { /*
        boolean firstTime = true;
        String test = "";
        for (ColorExpression element : colors) {
            if (firstTime) {

            }
        }
        */


        String res = "(" + colors.get(0);
        for (int i = 1; i < colors.size(); i++) {
            res += ", " + colors.get(i).toString();
        }
        res += ")";
        return res;
    }

    @Override
    public ExprStringPosition[] getChildren() {
        ExprStringPosition[] children = new ExprStringPosition[colors.size()];

        int i = 0;
        int endPrev = 0;
        for (ColorExpression p : colors) {

            int start = 1;
            int end = 0;

            if (i == 0) {
                end = start + p.toString().length();
                endPrev = end;

            } else {
                start = endPrev + 2;
                end = start + p.toString().length();
                endPrev = end;
            }

            ExprStringPosition pos = new ExprStringPosition(start, end, p);

            children[i] = pos;
            i++;
        }

        return children;
    }
}
