package dk.aau.cs.verification;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map.Entry;

import dk.aau.cs.debug.Logger;
import dk.aau.cs.model.tapn.Colored.*;
import dk.aau.cs.model.tapn.Colored.Expressions.ArcExpression;
import jdk.vm.ci.meta.Local;
import pipe.dataLayer.DataLayer;
import pipe.dataLayer.NetType;
import pipe.gui.ExportBatchDialog;
import pipe.gui.graphicElements.Arc;
import pipe.gui.graphicElements.ArcPath;
import pipe.gui.graphicElements.ArcPathPoint;
import pipe.gui.graphicElements.Place;
import pipe.gui.graphicElements.PlaceTransitionObject;
import pipe.gui.graphicElements.Transition;
import pipe.gui.graphicElements.cpn.*;
import pipe.gui.graphicElements.tapn.TimedInhibitorArcComponent;
import pipe.gui.graphicElements.tapn.TimedInputArcComponent;
import pipe.gui.graphicElements.tapn.TimedOutputArcComponent;
import pipe.gui.graphicElements.tapn.TimedPlaceComponent;
import pipe.gui.graphicElements.tapn.TimedTransitionComponent;
import pipe.gui.graphicElements.tapn.TimedTransportArcComponent;
import dk.aau.cs.Messenger;
import dk.aau.cs.model.tapn.Bound;
import dk.aau.cs.model.tapn.IntBound;
import dk.aau.cs.model.tapn.LocalTimedPlace;
import dk.aau.cs.model.tapn.SharedPlace;
import dk.aau.cs.model.tapn.TimeInterval;
import dk.aau.cs.model.tapn.TimeInvariant;
import dk.aau.cs.model.tapn.TimedArcPetriNet;
import dk.aau.cs.model.tapn.TimedArcPetriNetNetwork;
import dk.aau.cs.model.tapn.TimedInhibitorArc;
import dk.aau.cs.model.tapn.TimedInputArc;
import dk.aau.cs.model.tapn.TimedOutputArc;
import dk.aau.cs.model.tapn.TimedPlace;
import dk.aau.cs.model.tapn.TimedToken;
import dk.aau.cs.model.tapn.TimedTransition;
import dk.aau.cs.model.tapn.TransportArc;
import dk.aau.cs.util.Tuple;

public class TAPNComposer implements ITAPNComposer {
	private Messenger messenger;
	private boolean hasShownMessage = false;
	private boolean singleComponentNoPrefix = false; // if set to true then nets with only a single component have no prefix before places/transitions 

	private HashSet<String> processedSharedObjects;
	private HashMap<TimedArcPetriNet, DataLayer> guiModels;
	private DataLayer composedGuiModel;

	public TAPNComposer(Messenger messenger, HashMap<TimedArcPetriNet, DataLayer> guiModels, boolean singleComponentNoPrefix){
		this.messenger = messenger;
		
		HashMap<TimedArcPetriNet, DataLayer> newGuiModels = new HashMap<TimedArcPetriNet, DataLayer>();
		for(Entry<TimedArcPetriNet, DataLayer> entry : guiModels.entrySet()) {
			newGuiModels.put(entry.getKey(), entry.getValue().copy(entry.getKey()));
		}
		
		this.guiModels = newGuiModels;
		this.singleComponentNoPrefix = singleComponentNoPrefix;
	}
	
	public TAPNComposer(Messenger messenger, boolean singleComponentNoPrefix) {
		this.messenger = messenger;
		this.singleComponentNoPrefix = singleComponentNoPrefix;
	}
	
	public Tuple<TimedArcPetriNet, NameMapping> transformModel(TimedArcPetriNetNetwork model) {
		processedSharedObjects = new HashSet<String>();
		TimedArcPetriNet tapn = new TimedArcPetriNet("ComposedModel");
		ColoredArcPetriNet cpn = new ColoredArcPetriNet("ComposedModel");
		DataLayer guiModel;
		if (model.getNetType().equals(NetType.CPN)) {
			guiModel = new DataLayer(NetType.CPN);
		}
		else {
			guiModel = new DataLayer(NetType.TAPN);
		}

		NameMapping mapping = new NameMapping();
		hasShownMessage = false;

		double greatestWidth = 0,
			   greatestHeight = 0;
		if (this.guiModels != null) {
			for (TimedArcPetriNet tapn1 : model.activeTemplates()) {
                                if (isComponentEmpty(this.guiModels.get(tapn1))) {
                                        continue;
                                }
				greatestWidth = greatestWidth >= getRightmostObject(guiModels.get(tapn1)).getPositionX() ? greatestWidth : getRightmostObject(guiModels.get(tapn1)).getPositionX();
				greatestHeight = greatestHeight >= getLowestObject(guiModels.get(tapn1)).getPositionY() ? greatestHeight : getLowestObject(guiModels.get(tapn1)).getPositionY();
			}
		}
		if(model.isColored()) {
			createSharedPlaces(model, cpn, mapping, guiModel);
			createPlaces(model, cpn, mapping, guiModel, greatestWidth, greatestHeight);
			createTransitions(model, cpn, mapping, guiModel, greatestWidth, greatestHeight);
			createInputArcs(model, cpn, mapping, guiModel, greatestWidth, greatestHeight);
			createOutputArcs(model, cpn, mapping, guiModel, greatestWidth, greatestHeight);
			createTransportArcs(model, cpn, mapping, guiModel, greatestWidth, greatestHeight);
			createInhibitorArcs(model, cpn, mapping, guiModel, greatestWidth, greatestHeight);
		}else{
			createSharedPlaces(model, tapn, mapping, guiModel);
			createPlaces(model, tapn, mapping, guiModel, greatestWidth, greatestHeight);
			createTransitions(model, tapn, mapping, guiModel, greatestWidth, greatestHeight);
			createInputArcs(model, tapn, mapping, guiModel, greatestWidth, greatestHeight);
			createOutputArcs(model, tapn, mapping, guiModel, greatestWidth, greatestHeight);
			createTransportArcs(model, tapn, mapping, guiModel, greatestWidth, greatestHeight);
			createInhibitorArcs(model, tapn, mapping, guiModel, greatestWidth, greatestHeight);
		}
		
		// Set composed guiModel in the instance variable
		this.composedGuiModel = guiModel;

		if(model.isColored()) {
			return new Tuple<TimedArcPetriNet, NameMapping>(cpn, mapping);
		}else {
			return new Tuple<TimedArcPetriNet, NameMapping>(tapn, mapping);
		}
	}
	
	public DataLayer getGuiModel() {
		return this.composedGuiModel;
	}
	
	private Tuple<Integer, Integer> calculateComponentPosition(int netNumber) {
		Integer x = netNumber % 2;
		Integer y = (int) Math.floor(netNumber / 2d);
		
		return new Tuple<Integer, Integer>(x, y);		
	}
	
        private boolean isComponentEmpty(DataLayer guiModel) {
                return guiModel.getPlaces().length==0 && guiModel.getTransitions().length==0;
        }
        
	private PlaceTransitionObject getRightmostObject(DataLayer guiModel) {
		PlaceTransitionObject returnObject = null;
		
		for (PlaceTransitionObject currentObject : guiModel.getPlaces()) {
			if (returnObject == null || returnObject.getPositionX() < currentObject.getPositionX()) {
				returnObject = currentObject;
			}
		}
		
		for (PlaceTransitionObject currentObject : guiModel.getTransitions()) {
			if (returnObject == null || returnObject.getPositionX() < currentObject.getPositionX()) {
				returnObject = currentObject;
			}
		}
		
		return returnObject;
	}

	private PlaceTransitionObject getLowestObject(DataLayer guiModel) {
		PlaceTransitionObject returnObject = null;
		
		for (PlaceTransitionObject currentObject : guiModel.getPlaces()) {
			if (returnObject == null || returnObject.getPositionY() < currentObject.getPositionY()) {
				returnObject = currentObject;
			}
		}
		
		for (PlaceTransitionObject currentObject : guiModel.getTransitions()) {
			if (returnObject == null || returnObject.getPositionY() < currentObject.getPositionY()) {
				returnObject = currentObject;
			}
		}
		
		return returnObject;
	}
	
	private Place getSharedPlace(String name) {
		for(Entry<TimedArcPetriNet, DataLayer> hashmapEntry : guiModels.entrySet()) {
			Place findPlace = hashmapEntry.getValue().getPlaceByName(name);
			if (findPlace != null)
				return findPlace;
		}
		return null;
	}
	
	private void createSharedPlaces(TimedArcPetriNetNetwork model, TimedArcPetriNet constructedModel, NameMapping mapping, DataLayer guiModel) {
		for(SharedPlace place : model.sharedPlaces()){
			if(!model.isSharedPlaceUsedInTemplates(place))	continue;
			
			String uniquePlaceName = (!singleComponentNoPrefix || model.activeTemplates().size() > 1) ? composedPlaceName(place) : place.name(); 

			LocalColoredPlace localColoredPlace = null;
			LocalTimedPlace localTimedPlace = null;
			if (place.invariant().upperBound() instanceof Bound.InfBound) {
				if (place instanceof SharedColoredPlace) {
					ColorType ct = ((SharedColoredPlace)place).getColorType();
					localColoredPlace = new LocalColoredPlace(uniquePlaceName, ct);
				}else {
					localTimedPlace = new LocalTimedPlace(uniquePlaceName, place.invariant());
				}
			} else {
				localColoredPlace = new LocalColoredPlace(uniquePlaceName);
				localTimedPlace = new LocalTimedPlace(uniquePlaceName, new TimeInvariant(place.invariant().isUpperNonstrict(), new IntBound(place.invariant().upperBound().value())));
			}

			if(model.getNetType().equals(NetType.CPN)) {
				constructedModel.add(localColoredPlace);
			}else {
				constructedModel.add(localTimedPlace);
			}
			mapping.addMappingForShared(place.name(), uniquePlaceName);
			if(model.getNetType().equals(NetType.CPN)) {
				for(TimedToken token : place.tokens()) {
						Color tokenColor  = ((ColoredToken) token).getColor();
						ColoredToken coloredToken = new ColoredToken(place,tokenColor);
						constructedModel.marking().add(coloredToken);
					}
			}else {
				for (TimedToken token : place.tokens()) {
					localTimedPlace.addToken(new TimedToken(localTimedPlace, token.age()));
				}
			}
			
			// Gui work
			if (this.guiModels != null) {
				Place oldPlace = getSharedPlace(place.name());
				TimedPlaceComponent newPlace;
				if (oldPlace != null) {
					if(model.getNetType().equals(NetType.CPN)){
						newPlace = new ColoredPlaceComponent(
								oldPlace.getPositionX(),
								oldPlace.getPositionY(),
								oldPlace.getId(),
								uniquePlaceName,
								oldPlace.getNameOffsetX(),
								oldPlace.getNameOffsetY(),
								oldPlace.getMarkingOffsetXObject(),
								oldPlace.getMarkingOffsetYObject());

						newPlace.setGuiModel(guiModel);
						newPlace.setUnderlyingPlace(localColoredPlace);
						newPlace.setName(uniquePlaceName);
						guiModel.addPlace(newPlace);
					}else {
						 newPlace = new TimedPlaceComponent(
								oldPlace.getPositionX(),
								oldPlace.getPositionY(),
								oldPlace.getId(),
								uniquePlaceName,
								oldPlace.getNameOffsetX(),
								oldPlace.getNameOffsetY(),
								0,
								oldPlace.getMarkingOffsetXObject().doubleValue(),
								oldPlace.getMarkingOffsetYObject().doubleValue(),
								0
						);
						newPlace.setGuiModel(guiModel);
						newPlace.setUnderlyingPlace(localTimedPlace);
						newPlace.setName(uniquePlaceName);
						guiModel.addPlace(newPlace);
					}
				}
			}
		}
	}

	private void createPlaces(TimedArcPetriNetNetwork model, TimedArcPetriNet constructedModel, NameMapping mapping, DataLayer guiModel, double greatestWidth, double greatestHeight) {
	    int i = 0;
		LocalTimedPlace timePlace = null;
		LocalColoredPlace coloredPlace = null;
		for (TimedArcPetriNet tapn : model.activeTemplates()) {
			DataLayer currentGuiModel = null;
			if (this.guiModels != null) {
				currentGuiModel = this.guiModels.get(tapn);
                                if (isComponentEmpty(currentGuiModel)) { 
                                        continue;
                                }
                        }
			Tuple<Integer, Integer> offset = this.calculateComponentPosition(i);

			for (TimedPlace timedPlace : tapn.places()) {			
				if(!timedPlace.isShared()){
					String uniquePlaceName = (!singleComponentNoPrefix || model.activeTemplates().size() > 1) ? composedPlaceName(timedPlace) : timedPlace.name();

					if(timedPlace instanceof LocalColoredPlace) {
						ColorType ct = ((LocalColoredPlace) timedPlace).getColorType();

						coloredPlace = new LocalColoredPlace(uniquePlaceName, ct);
						constructedModel.add(coloredPlace);
						mapping.addMapping(tapn.name(), timedPlace.name(), uniquePlaceName);

						for (TimedToken token : timedPlace.tokens()) {
							Color tokenColor  = ((ColoredToken) token).getColor();
							ColoredToken coloredToken = new ColoredToken(coloredPlace,tokenColor);
							constructedModel.marking().add(coloredToken);
						}

					}
					else if (timedPlace instanceof LocalTimedPlace) {
						if (timedPlace.invariant().upperBound() instanceof Bound.InfBound) {
							timePlace = new LocalTimedPlace(uniquePlaceName, timedPlace.invariant());
						} else {
							timePlace = new LocalTimedPlace(uniquePlaceName, new TimeInvariant(timedPlace.invariant().isUpperNonstrict(), new IntBound(timedPlace.invariant().upperBound().value())));
						}
						constructedModel.add(timePlace);
						mapping.addMapping(tapn.name(), timedPlace.name(), uniquePlaceName);

						for (TimedToken token : timedPlace.tokens()) {
							timePlace.addToken(new TimedToken(timePlace, token.age()));
						}
					}
					// Gui work
					if (this.guiModels != null) {
					    TimedPlaceComponent newPlace;
                        Place oldPlace = currentGuiModel.getPlaceByName(timedPlace.name());
						if (model.getNetType().equals(NetType.CPN)) {
						    newPlace = new ColoredPlaceComponent(

                                    oldPlace.getPositionX() + offset.value1() * greatestWidth,
                                    oldPlace.getPositionY() + offset.value2() * greatestHeight,
                                    oldPlace.getId(),
                                    uniquePlaceName,
                                    oldPlace.getNameOffsetX(),
                                    oldPlace.getNameOffsetY(),
                                    oldPlace.getMarkingOffsetXObject().doubleValue(),
                                    oldPlace.getMarkingOffsetYObject().doubleValue()
                            );

						    newPlace.setGuiModel(guiModel);
                        } else {
                             newPlace = new TimedPlaceComponent(
                                    oldPlace.getPositionX() + offset.value1() * greatestWidth,
                                    oldPlace.getPositionY() + offset.value2() * greatestHeight,
                                    oldPlace.getId(),
                                    uniquePlaceName,
                                    oldPlace.getNameOffsetX(),
                                    oldPlace.getNameOffsetY(),
                                    0,
                                    oldPlace.getMarkingOffsetXObject().doubleValue(),
                                    oldPlace.getMarkingOffsetYObject().doubleValue(),
                                    0
                            );
                            newPlace.setGuiModel(guiModel);
                        }

						if (timedPlace instanceof LocalColoredPlace) {
							newPlace.setUnderlyingPlace(coloredPlace);
						} else {
							newPlace.setUnderlyingPlace(timedPlace);
						}
						newPlace.setName(uniquePlaceName);
						guiModel.addPlace(newPlace);
					}
				}
			}
			i++;
		}
	}

	private void createTransitions(TimedArcPetriNetNetwork model, TimedArcPetriNet constructedModel, NameMapping mapping, DataLayer guiModel, double greatestWidth, double greatestHeight) {
		int i = 0;
		ColoredTransition coloredTransition = null;
		TimedTransition transition = null;
		for (TimedArcPetriNet tapn : model.activeTemplates()) {
                        
			DataLayer currentGuiModel = null;
			if (this.guiModels != null) {
				currentGuiModel = this.guiModels.get(tapn);
                                if (isComponentEmpty(currentGuiModel)) { 
                                        continue;
                                }
			}
			Tuple<Integer, Integer> offset = this.calculateComponentPosition(i);
			
			for (TimedTransition timedTransition : tapn.transitions()) {
				if(!processedSharedObjects.contains(timedTransition.name())){
					
					// CAUTION: This if statement removes orphan transitions.
					//   This changes answers for e.g. DEADLOCK queries if
					//   support for such queries are added later.
					// ONLY THE IF SENTENCE SHOULD BE REMOVED. REST OF CODE SHOULD BE LEFT INTACT
					if(!timedTransition.isOrphan()){
						String uniqueTransitionName = "";
						if (!singleComponentNoPrefix || model.activeTemplates().size() > 1) {
							uniqueTransitionName = composedTransitionName(timedTransition);
						} else {
							uniqueTransitionName = timedTransition.name();
						}
						if(timedTransition instanceof ColoredTransition) {

							coloredTransition = new ColoredTransition(uniqueTransitionName, timedTransition.isUrgent(),
									((ColoredTransition) timedTransition).getGuard());
							constructedModel.add(coloredTransition);
						}
						else if(timedTransition instanceof TimedTransition) {
							transition = new TimedTransition(uniqueTransitionName, timedTransition.isUrgent());
							constructedModel.add(transition);
						}
						
						// Gui work
						if (this.guiModels != null) {
							Transition oldTransition = currentGuiModel.getTransitionByName(timedTransition.name());
							TimedTransitionComponent newTransition = null;
							if(model.isColored()) {
								newTransition = new ColoredTransitionComponent(
										oldTransition.getPositionX() + offset.value1() * greatestWidth,
										oldTransition.getPositionY() + offset.value2() * greatestHeight,
										oldTransition.getId(),
										uniqueTransitionName,
										oldTransition.getNameOffsetX(),
										oldTransition.getNameOffsetY(),
										true,
										false,
										oldTransition.getAngle(),
										0);
							}else {
								 newTransition = new TimedTransitionComponent(
										oldTransition.getPositionX() + offset.value1() * greatestWidth,
										oldTransition.getPositionY() + offset.value2() * greatestHeight,
										oldTransition.getId(),
										uniqueTransitionName,
										oldTransition.getNameOffsetX(),
										oldTransition.getNameOffsetY(),
										true,
										false,
										oldTransition.getAngle(),
										0);
							}
							newTransition.setGuiModel(guiModel);
							if(timedTransition instanceof ColoredTransition) {
								newTransition.setUnderlyingTransition(coloredTransition);
							}
							else {
								newTransition.setUnderlyingTransition(transition);
							}
							newTransition.setName(uniqueTransitionName);
							guiModel.addTransition(newTransition);
						}
						
						if(timedTransition.isShared()){
							String name = timedTransition.sharedTransition().name();
							processedSharedObjects.add(name);
							mapping.addMappingForShared(name, uniqueTransitionName);
						}else{
							mapping.addMapping(tapn.name(), timedTransition.name(), uniqueTransitionName);
						}
					}else{
						if(!hasShownMessage && !(ExportBatchDialog.isDialogVisible())){
							messenger.displayInfoMessage("There are orphan transitions (no incoming and no outgoing arcs) in the model."
									+ System.getProperty("line.separator") + "They will be removed before the verification.");
							hasShownMessage = true;
						}
						else if(ExportBatchDialog.isDialogVisible()) {
							ExportBatchDialog.setNoOrphanTransitions(true);
						}
					}
				}
			}
			i++;
		}
	}
	
	private ArcPath createArcPath(DataLayer currentGuiModel, PlaceTransitionObject source, PlaceTransitionObject target, Arc arc, double offsetX, double offsetY) {
		Arc guiArc = currentGuiModel.getArcByEndpoints(source, target);
		ArcPath arcPath = guiArc.getArcPath();
		int arcPathPointsNum = arcPath.getNumPoints();
		
		// Build ArcPath
		ArcPath newArcPath = new ArcPath(arc);
		for(int k = 0; k < arcPathPointsNum; k++) {
			ArcPathPoint point = arcPath.getArcPathPoint(k);
			newArcPath.addPoint(
					point.getPoint().x + offsetX,
					point.getPoint().y + offsetY,
					point.getPointType()
					);
		}
		
		return newArcPath;
	}

	private void createInputArcs(TimedArcPetriNetNetwork model, TimedArcPetriNet constructedModel, NameMapping mapping, DataLayer guiModel, double greatestWidth, double greatestHeight) {
		int i = 0;
		 TimedInputArc addedArc = null;
		 ColoredInputArc addedColoredArc = null;

		for (TimedArcPetriNet tapn : model.activeTemplates()) {
                        
			DataLayer currentGuiModel = null;
			if (this.guiModels != null) {
				currentGuiModel = this.guiModels.get(tapn);
                                if (isComponentEmpty(currentGuiModel)) { 
                                        continue;
                                }
                        }
			Tuple<Integer, Integer> offset = this.calculateComponentPosition(i);
			
			for (TimedInputArc arc : tapn.inputArcs()) {
				String sourceTemplate = arc.source().isShared() ? "" : tapn.name();
				TimedPlace source = constructedModel.getPlaceByName(mapping.map(sourceTemplate, arc.source().name()));
				
				String targetTemplate = arc.destination().isShared() ? "" : tapn.name();
				TimedTransition target = constructedModel.getTransitionByName(mapping.map(targetTemplate, arc.destination().name()));

				TimeInterval newInterval = null;
				List<ColoredTimeInterval> ctiList = new ArrayList<ColoredTimeInterval>();
				if (tapn.parentNetwork().isColored() && arc instanceof ColoredInputArc) {
					ctiList = ((ColoredInputArc) arc).getColorTimeIntervals();
				} else {
					newInterval = new TimeInterval(arc.interval());
					newInterval.setLowerBound(new IntBound(newInterval.lowerBound().value()));
					if (newInterval.upperBound() instanceof Bound.InfBound) {
						newInterval.setUpperBound(newInterval.upperBound());
					} else {
						newInterval.setUpperBound(new IntBound(newInterval.upperBound().value()));
					}
				}


				if(arc instanceof ColoredInputArc) {
					ArcExpression arcExpression = ((ColoredInputArc) arc).getArcExpression();
					addedColoredArc = new ColoredInputArc(source, target, ctiList, arcExpression);
					constructedModel.add(addedColoredArc);
				}
				else {
					addedArc = new TimedInputArc(source, target, newInterval, arc.getWeightValue());
					constructedModel.add(addedArc);
				}

				
				// Gui work
				if (this.guiModels != null) {
					Place guiSource = guiModel.getPlaceByName(mapping.map(sourceTemplate, arc.source().name()));
					Transition guiTarget = guiModel.getTransitionByName(mapping.map(targetTemplate, arc.destination().name()));
					ArcExpression arcExpression = ((ColoredInputArc) arc).getArcExpression();
					Place oldGuiSource = currentGuiModel.getPlaceByName(arc.source().name());
					Transition oldGuiTarget = currentGuiModel.getTransitionByName(arc.destination().name());

					if(arc instanceof ColoredInputArc) {
						Arc newColoredArc = new ColoredInputArcComponent(new ColoredOutputArcComponent(
								0d,
								0d,
								0d,
								0d,
								guiSource,
								guiTarget,
								arc.getWeight().value(),
								arcExpression,
								mapping.map(sourceTemplate, arc.source().name() + "_to_" + mapping.map(targetTemplate, arc.destination().name())),
								false));
						ArcPath newColoredArcPath = createArcPath(currentGuiModel, oldGuiSource, oldGuiTarget,
								newColoredArc, offset.value1() * greatestWidth, offset.value2() * greatestHeight);
						((ColoredInputArcComponent) newColoredArc).setUnderlyingArc(addedColoredArc);
						newColoredArc.setArcPath(newColoredArcPath);
						newColoredArc.setGuiModel(guiModel);
						newColoredArc.updateArcPosition();
						guiModel.addPetriNetObject(newColoredArc);
						guiSource.addConnectFrom(newColoredArc);
						guiTarget.addConnectTo(newColoredArc);
					}
					else {
						Arc newArc = new TimedInputArcComponent(new TimedOutputArcComponent(
								0d,
								0d,
								0d,
								0d,
								guiSource,
								guiTarget,
								arc.getWeight().value(),
								mapping.map(sourceTemplate, arc.source().name()) + "_to_" + mapping.map(targetTemplate, arc.destination().name()),
								false
						));
						ArcPath newArcPath = createArcPath(currentGuiModel, oldGuiSource, oldGuiTarget, newArc,
								offset.value1() * greatestWidth, offset.value2() * greatestHeight);
						((TimedInputArcComponent) newArc).setUnderlyingArc(addedArc);
						newArc.setArcPath(newArcPath);
						newArc.setGuiModel(guiModel);
						newArc.updateArcPosition();
						guiModel.addPetriNetObject(newArc);
						guiSource.addConnectFrom(newArc);
						guiTarget.addConnectTo(newArc);
					}
				}
			}
			i++;
		}
	}

	private void createOutputArcs(TimedArcPetriNetNetwork model, TimedArcPetriNet constructedModel, NameMapping mapping, DataLayer guiModel, double greatestWidth, double greatestHeight) {
		int i = 0;
		TimedOutputArc addedArc = null;
		ColoredOutputArc addedColoredArc = null;
		for (TimedArcPetriNet tapn : model.activeTemplates()) {
			DataLayer currentGuiModel = null;
			if (this.guiModels != null) {
				currentGuiModel = this.guiModels.get(tapn);
                                if (isComponentEmpty(currentGuiModel)) { 
                                        continue;
                                }
			}
			Tuple<Integer, Integer> offset = this.calculateComponentPosition(i);
			
			for (TimedOutputArc arc : tapn.outputArcs()) {
				String sourceTemplate = arc.source().isShared() ? "" : tapn.name();
				TimedTransition source = constructedModel.getTransitionByName(mapping.map(sourceTemplate, arc.source().name()));

				String destinationTemplate = arc.destination().isShared() ? "" : tapn.name();
				TimedPlace target = constructedModel.getPlaceByName(mapping.map(destinationTemplate, arc.destination().name()));

				Arc[] guiModelArc = guiModel.getArcs();
				for(int j = 0; j < guiModelArc.length; j++) {
					if(guiModelArc[j] instanceof ColoredOutputArcComponent) {
						if( ((ColoredOutputArcComponent)guiModelArc[j]).getExpression() != null) {
							ColoredOutputArcComponent coloredOutputArcComponent = (ColoredOutputArcComponent) guiModelArc[j];
							ArcExpression expr = coloredOutputArcComponent.getExpression();
						}
					}
					else if(guiModelArc[j] instanceof ColoredInputArcComponent) {
						if( ((ColoredInputArcComponent)guiModelArc[j]).getArcExpression() != null) {
							ColoredInputArcComponent coloredOutputArcComponent = (ColoredInputArcComponent) guiModelArc[j];
							ArcExpression expr = coloredOutputArcComponent.getArcExpression();
						}
					}
				}

				if(arc instanceof ColoredOutputArc) {
					ArcExpression expr = null;
					if( ((ColoredOutputArc) arc).getExpression() != null) {
						ColoredOutputArc colorArc = (ColoredOutputArc) arc;
						expr = colorArc.getExpression();
					}
					ArcExpression expression = ((ColoredOutputArc) arc).getExpression();
					addedColoredArc = new ColoredOutputArc(source, target, expression);
					constructedModel.add(addedColoredArc);
				}
				else {
					addedArc = new TimedOutputArc(source, target, arc.getWeightValue());
					constructedModel.add(addedArc);
				}
				
				// Gui work
				if (this.guiModels != null) {
					Transition guiSource = guiModel.getTransitionByName(mapping.map(sourceTemplate, arc.source().name()));
					Place guiTarget = guiModel.getPlaceByName(mapping.map(destinationTemplate, arc.destination().name()));
					ArcExpression expression = ((ColoredOutputArc) arc).getExpression();
					Transition oldGuiSource = currentGuiModel.getTransitionByName(arc.source().name());
					Place oldGuiTarget = currentGuiModel.getPlaceByName(arc.destination().name());

					if(arc instanceof ColoredOutputArc) {
						ColoredOutputArcComponent newColoredArc = new ColoredOutputArcComponent(
								0d,
								0d,
								0d,
								0d,
								guiModel.getTransitionByName(mapping.map(sourceTemplate, arc.source().name())),
								guiModel.getPlaceByName(mapping.map(destinationTemplate, arc.destination().name())),
								arc.getWeight().value(),
								expression,
								mapping.map(sourceTemplate, arc.source().name()) + "_to_" + mapping.map(destinationTemplate, arc.destination().name()),
								false
						);
						ArcPath newColoredArcPath = createArcPath(currentGuiModel, oldGuiSource, oldGuiTarget,
								newColoredArc, offset.value1() * greatestWidth, offset.value2() * greatestHeight);
						newColoredArc.setUnderlyingArc(addedColoredArc);
						newColoredArc.setArcPath(newColoredArcPath);
						newColoredArc.setGuiModel(guiModel);
						newColoredArc.updateArcPosition();
						guiModel.addArc(newColoredArc);
						guiSource.addConnectTo(newColoredArc);
						guiTarget.addConnectFrom(newColoredArc);
					}
					else {
						TimedOutputArcComponent newArc = new TimedOutputArcComponent(
								0d,
								0d,
								0d,
								0d,
								guiModel.getTransitionByName(mapping.map(sourceTemplate, arc.source().name())),
								guiModel.getPlaceByName(mapping.map(destinationTemplate, arc.destination().name())),
								arc.getWeight().value(),
								mapping.map(sourceTemplate, arc.source().name()) + "_to_" + mapping.map(destinationTemplate, arc.destination().name()),
								false
						);
						ArcPath newArcPath = createArcPath(currentGuiModel, oldGuiSource, oldGuiTarget, newArc,
								offset.value1() * greatestWidth, offset.value2() * greatestHeight);
						newArc.setUnderlyingArc(addedArc);
						newArc.setArcPath(newArcPath);
						newArc.setGuiModel(guiModel);
						newArc.updateArcPosition();
						guiModel.addArc(newArc);
						guiSource.addConnectTo(newArc);
						guiTarget.addConnectFrom(newArc);
					}
				}
			}
			i++;
		}
	}

	private void createTransportArcs(TimedArcPetriNetNetwork model, TimedArcPetriNet constructedModel, NameMapping mapping, DataLayer guiModel, double greatestWidth, double greatestHeight) {
		int i = 0;
		int nextGroupNr = 0;
		TransportArc addedArc = null;
		ColoredTransportArc addedColoredArc = null;
		for (TimedArcPetriNet tapn : model.activeTemplates()) {
                        
			DataLayer currentGuiModel = null;
			if (this.guiModels != null) {
				currentGuiModel = this.guiModels.get(tapn);
                                if (isComponentEmpty(currentGuiModel)) { 
                                        continue;
                                }
			}
			Tuple<Integer, Integer> offset = this.calculateComponentPosition(i);
			
			for (TransportArc arc : tapn.transportArcs()) {
				String sourceTemplate = arc.source().isShared() ? "" : tapn.name();
				String transitionTemplate = arc.transition().isShared() ? "" : tapn.name();
				String destinationTemplate = arc.destination().isShared() ? "" : tapn.name();

				TimedPlace source = constructedModel.getPlaceByName(mapping.map(sourceTemplate, arc.source().name()));
				TimedTransition transition = constructedModel.getTransitionByName(mapping.map(transitionTemplate, arc.transition().name()));
				TimedPlace destination = constructedModel.getPlaceByName(mapping.map(destinationTemplate, arc.destination().name()));
				
				TimeInterval newInterval = new TimeInterval(arc.interval());
				List<ColoredTimeInterval> ctiList = new ArrayList<ColoredTimeInterval>();
				if (tapn.parentNetwork().isColored() && arc instanceof ColoredTransportArc) {
					ctiList = ((ColoredTransportArc) arc).getCtiList();
				}else {
					newInterval.setLowerBound(new IntBound(newInterval.lowerBound().value()));
					if (newInterval.upperBound() instanceof Bound.InfBound) {
						newInterval.setUpperBound(newInterval.upperBound());
					} else {
						newInterval.setUpperBound(new IntBound(newInterval.upperBound().value()));
					}
				}

				if(arc instanceof ColoredTransportArc) {
					addedColoredArc = new ColoredTransportArc(source, transition, destination, ctiList);
					constructedModel.add(addedColoredArc);
				}else {
					addedArc = new TransportArc(source, transition, destination, newInterval, arc.getWeightValue());
					constructedModel.add(addedArc);
				}
				
				//Create input transport arc
				// Gui work
				if (this.guiModels != null) {
					Place guiSourceIn = guiModel.getPlaceByName(mapping.map(sourceTemplate, arc.source().name()));
					Transition guiTargetIn = guiModel.getTransitionByName(mapping.map(transitionTemplate, arc.transition().name()));
					ArcExpression arcExpression = ((ColoredTransportArc) arc).getExpression();

					if (arc instanceof ColoredTransportArc) {
						ColoredTransportArcComponent newInArc = new ColoredTransportArcComponent(
								new ColoredInputArcComponent(new ColoredOutputArcComponent(
										0d,
										0d,
										0d,
										0d,
										guiSourceIn,
										guiTargetIn,
										arc.getWeight().value(),
										arcExpression,
										mapping.map(sourceTemplate, arc.source().name() + " _to_" + mapping.map(destinationTemplate, arc.destination().name())),
										false

								)),
								nextGroupNr,
								true
						);
						// Build ArcPath
						Place oldGuiSourceIn = currentGuiModel.getPlaceByName(arc.source().name());
						Transition oldGuiTargetIn = currentGuiModel.getTransitionByName(arc.transition().name());
						ArcPath newArcPathIn = createArcPath(currentGuiModel, oldGuiSourceIn, oldGuiTargetIn, newInArc, offset.value1() * greatestWidth, offset.value2() * greatestHeight);

						newInArc.setUnderlyingArc(addedColoredArc);
						newInArc.setArcPath(newArcPathIn);
						newInArc.setGuiModel(guiModel);
						newInArc.updateArcPosition();
						guiModel.addArc(newInArc);

						guiSourceIn.addConnectTo(newInArc);
						guiTargetIn.addConnectFrom(newInArc);

						// Calculate the next group number for this transport arc
						// By looking at the target of the newInArc -> a transition
						// Then finding the largest existing group number of outgoing transport arcs from this transition
						for (Object pt : newInArc.getTarget().getPostset()) {
							if (pt instanceof ColoredTransportArcComponent) {
								if (((ColoredTransportArcComponent) pt).getGroupNr() > nextGroupNr) {
									nextGroupNr = ((ColoredTransportArcComponent) pt).getGroupNr();
								}
							}
						}

						((ColoredTransportArcComponent) newInArc).setGroupNr(nextGroupNr + 1);

						//Create output transport arc
						Transition guiSourceOut = guiModel.getTransitionByName(mapping.map(transitionTemplate, arc.transition().name()));
						Place guiTargetOut = guiModel.getPlaceByName(mapping.map(destinationTemplate, arc.destination().name()));

						ColoredTransportArcComponent newOutArc = new ColoredTransportArcComponent(
								new ColoredInputArcComponent(new ColoredOutputArcComponent(
										0d,
										0d,
										0d,
										0d,
										guiSourceOut,
										guiTargetOut,
										1,
										null,
										mapping.map(transitionTemplate, arc.transition().name()) + "_to_" + mapping.map(destinationTemplate, arc.destination().name()),
										false
								)),
								nextGroupNr + 1,
								false);
						Transition oldGuiSourceOut = currentGuiModel.getTransitionByName(arc.transition().name());
						Place oldGuiTargetOut = currentGuiModel.getPlaceByName(arc.destination().name());
						ArcPath newArcPathOut = createArcPath(currentGuiModel, oldGuiSourceOut, oldGuiTargetOut, newOutArc, offset.value1() * greatestWidth, offset.value2() * greatestHeight);

						newOutArc.setUnderlyingArc(addedColoredArc);
						newOutArc.setArcPath(newArcPathOut);
						newOutArc.setGuiModel(guiModel);
						newOutArc.updateArcPosition();
						guiModel.addArc(newOutArc);

						// Add connection references to the two transport arcs
						newInArc.setConnectedTo(newOutArc);
						newOutArc.setConnectedTo(newInArc);

						guiSourceOut.addConnectTo(newOutArc);
						guiTargetOut.addConnectFrom(newOutArc);

					} else {
						TimedTransportArcComponent newInArc = new TimedTransportArcComponent(
								new TimedInputArcComponent(new TimedOutputArcComponent(
										0d,
										0d,
										0d,
										0d,
										guiSourceIn,
										guiTargetIn,
										arc.getWeight().value(),
										mapping.map(sourceTemplate, arc.source().name()) + "_to_" + mapping.map(transitionTemplate, arc.transition().name()),
										false
								)),
								nextGroupNr,
								true
						);

						// Build ArcPath
						Place oldGuiSourceIn = currentGuiModel.getPlaceByName(arc.source().name());
						Transition oldGuiTargetIn = currentGuiModel.getTransitionByName(arc.transition().name());
						ArcPath newArcPathIn = createArcPath(currentGuiModel, oldGuiSourceIn, oldGuiTargetIn, newInArc, offset.value1() * greatestWidth, offset.value2() * greatestHeight);

						newInArc.setUnderlyingArc(addedArc);
						newInArc.setArcPath(newArcPathIn);
						newInArc.setGuiModel(guiModel);
						newInArc.updateArcPosition();
						guiModel.addArc(newInArc);

						guiSourceIn.addConnectTo(newInArc);
						guiTargetIn.addConnectFrom(newInArc);


						// Calculate the next group number for this transport arc
						// By looking at the target of the newInArc -> a transition
						// Then finding the largest existing group number of outgoing transport arcs from this transition
						for (Object pt : newInArc.getTarget().getPostset()) {
							if (pt instanceof TimedTransportArcComponent) {
								if (((TimedTransportArcComponent) pt).getGroupNr() > nextGroupNr) {
									nextGroupNr = ((TimedTransportArcComponent) pt).getGroupNr();
								}
							}
						}

						((TimedTransportArcComponent) newInArc).setGroupNr(nextGroupNr + 1);


						//Create output transport arc
						Transition guiSourceOut = guiModel.getTransitionByName(mapping.map(transitionTemplate, arc.transition().name()));
						Place guiTargetOut = guiModel.getPlaceByName(mapping.map(destinationTemplate, arc.destination().name()));

						TimedTransportArcComponent newOutArc = new TimedTransportArcComponent(
								new TimedInputArcComponent(new TimedOutputArcComponent(
										0d,
										0d,
										0d,
										0d,
										guiSourceOut,
										guiTargetOut,
										1,
										mapping.map(transitionTemplate, arc.transition().name()) + "_to_" + mapping.map(destinationTemplate, arc.destination().name()),
										false
								)),
								nextGroupNr + 1,
								false
						);

						// Build ArcPath
						Transition oldGuiSourceOut = currentGuiModel.getTransitionByName(arc.transition().name());
						Place oldGuiTargetOut = currentGuiModel.getPlaceByName(arc.destination().name());
						ArcPath newArcPathOut = createArcPath(currentGuiModel, oldGuiSourceOut, oldGuiTargetOut, newOutArc, offset.value1() * greatestWidth, offset.value2() * greatestHeight);

						newOutArc.setUnderlyingArc(addedArc);
						newOutArc.setArcPath(newArcPathOut);
						newOutArc.setGuiModel(guiModel);
						newOutArc.updateArcPosition();
						guiModel.addArc(newOutArc);

						// Add connection references to the two transport arcs
						newInArc.setConnectedTo(newOutArc);
						newOutArc.setConnectedTo(newInArc);

						guiSourceOut.addConnectTo(newOutArc);
						guiTargetOut.addConnectFrom(newOutArc);
					}
				}
			}
			i++;
		}
	}

	
	
	private void createInhibitorArcs(TimedArcPetriNetNetwork model, TimedArcPetriNet constructedModel, NameMapping mapping, DataLayer guiModel, double greatestWidth, double greatestHeight) {
		int i = 0;
		for (TimedArcPetriNet tapn : model.activeTemplates()) {
                        
			DataLayer currentGuiModel = null;
			if (this.guiModels != null) {
				currentGuiModel = this.guiModels.get(tapn);
                                if (isComponentEmpty(currentGuiModel)) { 
                                        continue;
                                }
                        }
			Tuple<Integer, Integer> offset = this.calculateComponentPosition(i);
			
			for (TimedInhibitorArc arc : tapn.inhibitorArcs()) {
				String sourceTemplate = arc.source().isShared() ? "" : tapn.name();
				String destinationTemplate = arc.destination().isShared() ? "" : tapn.name();
				
				TimedPlace source = constructedModel.getPlaceByName(mapping.map(sourceTemplate, arc.source().name()));
				TimedTransition target = constructedModel.getTransitionByName(mapping.map(destinationTemplate, arc.destination().name()));

				TimeInterval newInterval = new TimeInterval(arc.interval());
				newInterval.setLowerBound(new IntBound(newInterval.lowerBound().value()));
				if (newInterval.upperBound() instanceof Bound.InfBound) {
					newInterval.setUpperBound(newInterval.upperBound());
				} else {
					newInterval.setUpperBound(new IntBound(newInterval.upperBound().value()));
				}
				TimedInhibitorArc addedArc = new TimedInhibitorArc(source, target, newInterval, arc.getWeightValue());
				constructedModel.add(addedArc);
				
				// Gui work
				if (this.guiModels != null) {
					Place guiSource = guiModel.getPlaceByName(mapping.map(sourceTemplate, arc.source().name()));
					Transition guiTarget = guiModel.getTransitionByName(mapping.map(destinationTemplate, arc.destination().name()));
					Arc newArc = new TimedInhibitorArcComponent(new TimedOutputArcComponent(
							0d,
							0d,
							0d,
							0d,
							guiSource,
							guiTarget,
							arc.getWeight().value(),
							mapping.map(sourceTemplate, arc.source().name()) + "_to_" + mapping.map(destinationTemplate, arc.destination().name()),
							false
							), "");
					
					// Build ArcPath
					Place oldGuiSource = currentGuiModel.getPlaceByName(arc.source().name());
					Transition oldGuiTarget = currentGuiModel.getTransitionByName(arc.destination().name());
					ArcPath newArcPath = createArcPath(currentGuiModel, oldGuiSource, oldGuiTarget, newArc, offset.value1() * greatestWidth, offset.value2() * greatestHeight);
					
					((TimedInhibitorArcComponent) newArc).setUnderlyingArc(addedArc);
					newArc.setArcPath(newArcPath);
					newArc.setGuiModel(guiModel);
					newArc.updateArcPosition();
					guiModel.addPetriNetObject(newArc);
					guiSource.addConnectTo(newArc);
					guiTarget.addConnectFrom(newArc);
				}
			}
			i++;
		}
	}

   public String composedTransitionName(TimedTransition transition) {
        return  transition.isShared() ? "Shared_" + transition.name() : transition.model().name() + "_" + transition.name();
   }
   
   public String composedPlaceName(TimedPlace place) {
        return  place.isShared() ? "Shared_" + place.name() : ((LocalTimedPlace)place).model().name() + "_" + place.name();
   }
   

}
