package pipe.gui.graphicElements.cpn;

import dk.aau.cs.gui.Context;
import dk.aau.cs.gui.undo.Command;
import dk.aau.cs.model.tapn.Bound;
import dk.aau.cs.model.tapn.Colored.*;
import dk.aau.cs.model.tapn.Colored.Color;
import dk.aau.cs.model.tapn.ConstantBound;
import dk.aau.cs.model.tapn.TimedPlace;
import dk.aau.cs.model.tapn.TimedToken;
import jdk.vm.ci.meta.Local;
import pipe.gui.CreateGui;
import pipe.gui.Pipe;
import pipe.gui.graphicElements.tapn.TimedPlaceComponent;
import pipe.gui.undo.colored.ColoredPlaceInvariantEdit;
import pipe.gui.widgets.cpn.ColoredPlaceEditorPanel;
import pipe.gui.widgets.EscapableDialog;

import javax.swing.*;
import java.awt.*;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.List;
import java.util.Locale;
import java.util.Vector;

public class ColoredPlaceComponent extends TimedPlaceComponent {

    public ColoredPlaceComponent(double positionXInput, double positionYInput, String idInput, String nameInput,
                                 Double nameOffsetXInput, Double nameOffsetYInput, double markingOffsetXInput,
                                 double markingOffsetYInput) {
        //initialMarking and capacity is unused in parent, so we chose to pass 0
        super(positionXInput, positionYInput, idInput, nameInput, nameOffsetXInput, nameOffsetYInput, 0,
                markingOffsetXInput, markingOffsetYInput, 0);
    }

    public ColoredPlaceComponent(double positionXinput, double positionYInput, TimedPlace place) {
        super(positionXinput, positionYInput, place);
    }

    public ColoredTimeInvariant getInvariant() {
        return (ColoredTimeInvariant) place.invariant();
    }


    public Command setInvariant(ColoredTimeInvariant inv) {
        ColoredTimeInvariant old = (ColoredTimeInvariant) place.invariant();
        place.setInvariant(inv);
        update(true);

        return new ColoredPlaceInvariantEdit(this, old, inv);
    }

    @Override
    public ColoredPlaceComponent clone() {
        ColoredPlaceComponent toReturn = (ColoredPlaceComponent) super.clone();

        toReturn.setInvariant(this.getInvariant());
        return toReturn;

    }

    @Override
    public void update(boolean displayConstantNames) {
        if(place != null) {
            boolean gotTokens = false;
            String placeColorType = null;
            pnName.setName(place.name());
            if(place instanceof LocalColoredPlace) {
                if (((LocalColoredPlace) place).getColorType() != null) {
                    placeColorType = "[" + ((LocalColoredPlace) place).getColorType().getName() + "]";
                    gotTokens = true;
                }
            }
            else if(place instanceof SharedColoredPlace) {
                if(((SharedColoredPlace)place).getColorType() != null) {
                    placeColorType = "[" + ((SharedColoredPlace) place).getColorType().getName() + "]";
                    gotTokens = true;
                }
            }
            if ((place.invariant() != null) && !(place.invariant().upperBound() instanceof Bound.InfBound) && gotTokens) {
                pnName.setText("\nInv: " + place.invariant().toString(displayConstantNames) +
                                "\n" + placeColorType);
            }

            else if (gotTokens) {
                pnName.setText("\n" + placeColorType);
            }
            else{
                pnName.setText("");
            }

            if(place instanceof LocalColoredPlace) {
                if (((LocalColoredPlace) place).getCtiList() != null) {
                    String invariantText = "\n";
                    for (ColoredTimeInvariant timeInvariant : ((LocalColoredPlace) place).getCtiList()) {
                        if (timeInvariant != null)
                            invariantText += timeInvariant.toString() + "\n";
                    }
                    pnName.setText(pnName.getText() + invariantText);
                }
            }
            else if (place instanceof SharedColoredPlace){
                if (((SharedColoredPlace) place).getCtiList() != null) {
                    String invariantText = "\n";
                    for (ColoredTimeInvariant timeInvariant : ((SharedColoredPlace) place).getCtiList()) {
                        if (timeInvariant != null)
                            invariantText += timeInvariant.toString() + "\n";
                    }
                    pnName.setText(pnName.getText() + invariantText);
                }
            }

            // Handle constant highlighting
            boolean focusedConstant = false;
            if(place.invariant().upperBound() instanceof ConstantBound){
                if(((ConstantBound) place.invariant().upperBound()).constant().hasFocus()){
                    focusedConstant = true;
                }
            }
            if(focusedConstant){
                pnName.setForeground(Pipe.SELECTION_TEXT_COLOUR);
            }else{
                pnName.setForeground(Pipe.ELEMENT_TEXT_COLOUR);
            }

            pnName.displayName(attributesVisible);

        } else {
            pnName.setName("");
            pnName.setText("");
        }
        pnName.zoomUpdate(zoom);
        updateOnMoveOrZoom();
        repaint();
    }

    @SuppressWarnings("Duplicates")
    @Override
    protected void paintTokens(Graphics g) {
        DecimalFormat df = new DecimalFormat();
        df.setMinimumFractionDigits(1);
        df.setMaximumFractionDigits(1);
        df.setRoundingMode(RoundingMode.DOWN);

        Insets insets = getInsets();
        int x = insets.left;
        int y = insets.top;

        // int marking = getCurrentMarking();
        List<TimedToken> myTokens = place.tokens();
        int marking = place.numberOfTokens();

        // Decide whether or not to draw tokens as dots
        boolean drawDots = false;
        if(!CreateGui.getApp().showTokenAge()) {
            // only draw dots if there're 1-5 tokens.
            drawDots = (marking > 0 && marking < 6);

            // if any token's age is > 0, do not draw dots
            for (TimedToken token : myTokens) {
                if (token.age().compareTo(BigDecimal.valueOf(0)) != 0) {
                    drawDots = false;
                    break;
                }
            }
        }

        // structure sees how many markings there are and fills the place in
        // with the appropriate number or tokens.
        if(drawDots) {
            int xPos, yPos;
            switch(marking){
                case 5:
                    g.fillOval(x + tMiddleX, y + tMiddleY, tWidth, tHeight); // middle
                    /* falls through */
                case 4:
                    g.fillOval(x + tLeftX, y + tTopY, tWidth, tHeight); // top left
                    /* falls through */
                case 3:
                    if (marking == 5 || marking == 4) {
                        xPos = x + tRightX;	// top right
                        yPos = y + tTopY;
                    } else {
                        xPos = x + tLeftX;		// top left
                        yPos = y + tTopY;
                    }
                    g.fillOval(xPos, yPos, tWidth, tHeight);
                    /* falls through */
                case 2:
                    if (marking == 5 || marking == 4){
                        xPos = x + tLeftX;       // bottom left
                        yPos = y + tBotY;
                    } else if (marking == 3){
                        xPos = x + tMiddleX;   // middle
                        yPos = y + tMiddleY;
                    } else {
                        xPos = x + tLeftX;       // left middle
                        yPos = y + tMiddleY;
                    }
                    g.fillOval(xPos, yPos, tWidth, tHeight);
                    /* falls through */
                case 1:
                    if(marking == 5 || marking == 4 || marking == 3){
                        xPos = x + tRightX;    // bottom right
                        yPos = y + tBotY;
                    } else if (marking == 2){
                        xPos = x + tRightX;    // right middle
                        yPos = y + tMiddleY;
                    } else {
                        xPos = x + tMiddleX; // middle
                        yPos = y + tMiddleY;
                    }
                    g.fillOval(xPos, yPos, tWidth, tHeight);
                case 0:
                    break;
            }
        } else {	// print token age
            switch (marking) {
                case 0:
                    break;
                default:
                    if (marking > 999) {
                        // XXX could be better...
                        g.drawString("#" + String.valueOf(marking), x, y + 20);
                    } else if (marking > 99) {
                        g.drawString("#" + String.valueOf(marking), x, y + 20);
                    } else if (marking > 9) {
                        g.drawString("#" + String.valueOf(marking), x + 2, y + 20);
                    } else {
                        g.drawString("#" + String.valueOf(marking), x + 6, y + 20);
                    }
                    break;
            }
        }
    }

    @Override
    public String getStringOfTokens() {
        StringBuffer buffer = new StringBuffer("{");
        DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(Pipe.AGE_DECIMAL_PRECISION);
        df.setDecimalFormatSymbols(new DecimalFormatSymbols(Locale.ENGLISH));
        Vector<ColoredToken> tokens = new Vector(place.tokens());
        boolean first = true;
        for (ColoredToken token : tokens) {
            if (!first)
                buffer.append(", ");
            buffer.append("(" + df.format(token.age()) + ", " + token.getColor().getColorName() + ")");

            first = false;
        }
        buffer.append('}');

        return buffer.toString();
    }
    @Override
    public void addTokens(int numberOfTokensToAdd) {
        ColorType colorType = ((LocalColoredPlace)this.underlyingPlace()).getColorType();
        Color color = colorType.getFirstColor();
        for (int i = 0; i < numberOfTokensToAdd; i++) {
            place.addToken(new ColoredToken(place, color, BigDecimal.ZERO));
        }
    }

    @Override
    public void showEditor() {
        EscapableDialog guiDialog = new EscapableDialog(CreateGui.getApp(), "Edit Place", true);
        Container contentPane = guiDialog.getContentPane();

        // 1 Set layout
        contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));

        // 2 Add Place editor
        if(this.underlyingPlace() instanceof LocalColoredPlace) {
            LocalColoredPlace place = (LocalColoredPlace) this.underlyingPlace();
            ColoredPlaceEditorPanel placeEditor;
            if (place.getColorType() != null) {
                placeEditor = new ColoredPlaceEditorPanel(guiDialog.getRootPane(), this, place.getColorType(), new Context(CreateGui.getCurrentTab()));

            } else {
                placeEditor = new ColoredPlaceEditorPanel(guiDialog.getRootPane(), this, new Context(CreateGui.getCurrentTab()));
            }
            // placeEditor.setMaximumSize(new Dimension(400, 600));
            JScrollPane scrollPane = new JScrollPane(placeEditor);
            scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
            scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
            // scrollPane.setMaximumSize(new Dimension(800, 600));
            contentPane.add(scrollPane);
        }
        else if (this.underlyingPlace() instanceof SharedColoredPlace) {
            SharedColoredPlace place = (SharedColoredPlace) this.underlyingPlace();
            ColoredPlaceEditorPanel placeEditor;
            if (place.getColorType() != null) {
                placeEditor = new ColoredPlaceEditorPanel(guiDialog.getRootPane(), this, place.getColorType(), new Context(CreateGui.getCurrentTab()));

            } else {
                placeEditor = new ColoredPlaceEditorPanel(guiDialog.getRootPane(), this, new Context(CreateGui.getCurrentTab()));
            }
            // placeEditor.setMaximumSize(new Dimension(400, 600));
            JScrollPane scrollPane = new JScrollPane(placeEditor);
            scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
            scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
            // scrollPane.setMaximumSize(new Dimension(800, 600));
            contentPane.add(scrollPane);
        }
        guiDialog.setResizable(true);

        // Make window fit contents' preferred size
        guiDialog.pack();

        // Move window to the middle of the screen
        guiDialog.setLocationRelativeTo(null);
        guiDialog.setVisible(true);
    }
}
