package pipe.gui.widgets;

import dk.aau.cs.gui.Context;
import dk.aau.cs.model.tapn.Colored.*;
import dk.aau.cs.model.tapn.Colored.Color;
import dk.aau.cs.model.tapn.Colored.ExpressionSupport.ExprValues;
import dk.aau.cs.model.tapn.TimedInputArc;
import dk.aau.cs.model.tapn.TimedOutputArc;
import dk.aau.cs.model.tapn.TimedPlace;
import dk.aau.cs.model.tapn.TimedTransition;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

public class TypeCheckingPanel extends JPanel {
    JRootPane rootPane;
    Context context;
    List<ColoredInputArc> inputArcs = new ArrayList();
    List<ColoredOutputArc> outputArcs = new ArrayList();
    List<ColoredTransition> transitions = new ArrayList();
    List<LocalColoredPlace> places = new ArrayList();
    List<String> errorMsgs = new ArrayList<String>();

    public TypeCheckingPanel(JRootPane rootPane, Context context) {
        this.context = context;
        this.rootPane = rootPane;
        runTypeCheckingAnalysis();
        setupPanel();
    }

    private void runTypeCheckingAnalysis() {
        loadExpressions();
        for (ColoredInputArc inputArc : inputArcs) {
            if (inputArc.getArcExpression() != null) {
                getColorTypeMismatchErrors(inputArc.getArcExpression().getValues(new ExprValues()), "Input arc from place " + inputArc.source().name() + " to transition " + inputArc.destination().name() );
                getConnectedPlaceErrorsInput(inputArc, inputArc.getArcExpression().getValues(new ExprValues()), "Input arc from place " + inputArc.source().name() + " to transition " + inputArc.destination().name());
            }
        }
        for (ColoredOutputArc outputArc : outputArcs) {
            if (outputArc.getExpression() != null) {
                getColorTypeMismatchErrors(outputArc.getExpression().getValues(new ExprValues()), "Output arc from transition " + outputArc.source().name() + " to place " + outputArc.destination().name());
                getConnectedPlaceErrosOutput(outputArc, outputArc.getExpression().getValues(new ExprValues()), "outputArc from transition " + outputArc.source().name() + " to place " + outputArc.destination().name());
            }
        }

        for (ColoredTransition transition : transitions) {
            if (transition.getGuard() != null) {
                getColorTypeMismatchErrors(transition.getGuard().getValues(new ExprValues()), "Transition " + transition.name());
            }
        }

    }

    private void getConnectedPlaceErrosOutput(ColoredOutputArc outputArc, ExprValues exprValues, String identifier) {
        ColorType colorType = ((LocalColoredPlace)outputArc.destination()).getColorType();
        for (Variable variable : exprValues.getVariables()) {
            if (variable.getColorType().getName().equals(colorType.getName())) {
                errorMsgs.add("<html>" + identifier + ": The Variable <b>" + variable.getName() + "</b> does not have the same color type as the connecting place.");
            }
        }
        for (Color color : exprValues.getColors()) {
            boolean colorExists = false;
            for (Color color1 : colorType) {
                if (color.getName().equals(color1.getColorName())) {
                    colorExists = true;
                }
            }
            if (colorExists != true) {
                errorMsgs.add("<html>" + identifier + ": The color <b>" + color.getName() + "</b> beloning to color type <b>" + color.getColorType() + "</b> does not exist in the color type to the connecting place");
            }
        }

        for (ColorType type : exprValues.getColorTypes()) {
            if (!type.getName().equals(colorType.getName())) {
                errorMsgs.add("<html>" + identifier + ": The color type <b>" + colorType.getName() + "</b> does not have the same color type as the connecting place.");
            }
        }
    }

    private void getConnectedPlaceErrorsInput(ColoredInputArc inputArc, ExprValues exprValues, String identifier) {
        ColorType colorType = ((LocalColoredPlace)inputArc.source()).getColorType();
        for (Variable variable : exprValues.getVariables()) {
            if (variable.getColorType().getName().equals(colorType.getName())) {
                errorMsgs.add("<html>" + identifier + ": The Variable <b>" + variable.getName() + "</b> does not have the same color type as the connecting place.");
            }
        }
        for (Color color : exprValues.getColors()) {
            boolean colorExists = false;
            for (Color color1 : colorType) {
                if (color.getName().equals(color1.getColorName())) {
                    colorExists = true;
                }
            }
            if (colorExists != true) {
                errorMsgs.add("<html>" + identifier + ": The color <b>" + color.getName() + "</b> beloning to color type <b>" + color.getColorType() + "</b> does not exist in the color type to the connecting place");
            }
        }

        for (ColorType type : exprValues.getColorTypes()) {
            if (!type.getName().equals(colorType.getName())) {
                errorMsgs.add("<html>" + identifier + ": The color type <b>" + colorType.getName() + "</b> does not have the same color type as the connecting place.");
            }
        }
    }

    private void getColorTypeMismatchErrors(ExprValues exprValues, String identifier) {
        for (Color color : exprValues.getColors()) {
            boolean colorTypeExists = false;
            boolean colorExists = false;

            ColorType colorColorType = color.getColorType();
            for (ColorType colorType : exprValues.getColorTypes()) {
                if (!colorType.getName().equals(colorColorType.getName())) {
                    errorMsgs.add("<html>" + identifier + ": The color <b>" + color.getColorName() + "</b> does not have the same color type as <b>" + colorType.getName() + "</b></html>");
                }
            }
            for (Variable variable : exprValues.getVariables()) {
                if (!variable.getColorType().getName().equals(colorColorType.getName())) {
                    errorMsgs.add("<html>" + identifier + ": The color <b>" + color.getColorName() + "</b> does not have the same color type as the variable <b>" + variable.getName() + "</b></html>");
                }
            }

            for (Color exprValuesColor : exprValues.getColors()) {
                if (!exprValuesColor.equals(colorColorType)) { // we dont want to check the object against it self (we iterate on the same list twice)
                    if (!exprValuesColor.getColorType().getName().equals(colorColorType.getName())) {
                        errorMsgs.add("<html>" + identifier + ": The color <b>" + exprValuesColor.getName() + " </b>and the color <b>" + color.getColorName() + "</b> does not have the same color type."  + "</html>");
                    }
                }
            }

            for (ColorType colorType : context.network().colorTypes()) {
                if (!(colorType instanceof ProductType)) {
                    if (colorType.getName().equals(colorColorType.getName())) {
                        colorTypeExists = true;
                    }
                }
                for (Color color1 : colorType) {
                    if (color1.getName().equals(color.getColorName())) {
                        colorExists = true;
                    }
                }
            }
            if (colorTypeExists != true) {
                errorMsgs.add("<html>" + identifier + ": The color type, the color <b>" + color.getName() + "</b> belongs to does not exists anymore</html>");
            }
            else if (colorExists != true) {
                errorMsgs.add("<html" + identifier + ": The color <b>" + color.getName() + " </b>does not exists in any color types</html>");
            }
        }

        for (Variable variable : exprValues.getVariables()) {
            boolean doesVariableExist = false;
            for (Variable variable1 : context.network().variables()) {
                if (variable.getName().equals(variable1.getName()) && variable.getColorType().getName().equals(variable1.getColorType().getName())) {
                    doesVariableExist = true;
                }
            }
            if (doesVariableExist != true) {
                errorMsgs.add("<html>" + identifier + ": The variable <b>" + variable + " </b>does not match any variable in the model and the expression has to be updated</html>");            }
        }
    }

    private void loadExpressions() {
        inputArcs.clear();
        outputArcs.clear();
        transitions.clear();
        places.clear();

        for (TimedInputArc arc : context.activeModel().inputArcs()) {
            inputArcs.add((ColoredInputArc) arc);
        }
        for (TimedOutputArc outputArc : context.activeModel().outputArcs()) {
            outputArcs.add((ColoredOutputArc) outputArc);
        }
        for (TimedTransition transition : context.activeModel().transitions()) {
            transitions.add((ColoredTransition)transition);
        }
        for (TimedPlace place : context.activeModel().places()) {
            places.add((LocalColoredPlace) place);
        }
    }

    private void setupPanel() {
        JPanel typeCheckingListPanel =new JPanel();
        typeCheckingListPanel.setLayout(new GridBagLayout());

      //  typeCheckingListPanel.setPreferredSize(new Dimension(300, 300));

        DefaultListModel listModel = new DefaultListModel();
        JList errorsList = new JList(listModel);
        JScrollPane errorsListScrollPane = new JScrollPane(errorsList);
        errorsListScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
        errorsListScrollPane.setPreferredSize(new Dimension(600, 200));
        errorsListScrollPane.setMaximumSize(new Dimension(600, 200));
        errorsListScrollPane.setMinimumSize(new Dimension(600, 200));
        errorsListScrollPane.setBorder(BorderFactory.createTitledBorder( "Errors List"));

        JButton runAnalysisButton = new JButton();
        runAnalysisButton.setPreferredSize(new Dimension(150, 30));
        runAnalysisButton.setText("Run Analysis");

        JButton OKButton = new JButton();
        OKButton.setPreferredSize(new Dimension(100, 30));
        OKButton.setText("OK");

        for (String errorMsg : errorMsgs) {
            listModel.addElement(errorMsg);
        }

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridwidth = 2;
        gbc.gridy = 0;
        typeCheckingListPanel.add(errorsListScrollPane, gbc);

        gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 1;
        gbc.anchor = GridBagConstraints.SOUTHEAST;
        typeCheckingListPanel.add(runAnalysisButton, gbc);

        runAnalysisButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                runTypeCheckingAnalysis();
            }
        });

        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        add(typeCheckingListPanel, gbc);

        gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 1;
        gbc.anchor = GridBagConstraints.SOUTHEAST;
       // add(OKButton, gbc);
    }

}
