package pipe.gui.widgets.cpn;

import dk.aau.cs.debug.Logger;
import dk.aau.cs.gui.Context;
import dk.aau.cs.model.tapn.AbstractTimeConstraint;
import dk.aau.cs.model.tapn.Colored.*;
import dk.aau.cs.model.tapn.Colored.Color;
import org.jdesktop.swingx.JXComboBox;
import pipe.gui.CreateGui;
import pipe.gui.graphicElements.cpn.ColoredInputArcComponent;
import pipe.gui.graphicElements.cpn.ColoredPlaceComponent;
import pipe.gui.graphicElements.cpn.ColoredTransitionComponent;
import pipe.gui.graphicElements.cpn.ColoredTransportArcComponent;
import pipe.gui.widgets.EscapableDialog;
import sun.rmi.runtime.Log;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import java.util.Vector;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

public class ColorIntervalAndInvariantEditPanel extends JPanel {

    private Context context;
    private ColorType colorType;
    private boolean isInterval;
    private boolean isTransport = false;

    private JPanel nonDefaultColorPanel;
    private ColorComboboxPanel colorComboboxPanel;
    private JButton addTimeConstraintButton;
    private JButton removeTimeConstraintButton;
    private JButton editTimeConstraintButton;
    private DefaultListModel timeConstraintListModel;
    private JList timeConstraintList;
    private JScrollPane timeConstraintScrollPane;

    private ColoredPlaceComponent placeComp;
    private ColoredTransportArcComponent transComp;
    private ColoredInputArcComponent arcComp;

    public ColorIntervalAndInvariantEditPanel(Context context, ColorType colorType, boolean isInterval, ColoredPlaceComponent placeComp) {
        this.colorType = colorType;
        this.context = context;
        this.isInterval = isInterval;
        this.placeComp = placeComp;
        initPanel();
        setTimeConstraints();
    }

    public ColorIntervalAndInvariantEditPanel(Context context, ColorType colorType, boolean isInterval, ColoredInputArcComponent arcComp) {
        this.colorType = colorType;
        this.context = context;
        this.isInterval = isInterval;
        this.arcComp = arcComp;
        initPanel();
        setTimeConstraints();
    }

    public ColorIntervalAndInvariantEditPanel(Context context, ColorType colorType, boolean isInterval, ColoredTransportArcComponent transComp) {
        this.colorType = colorType;
        this.context = context;
        this.isInterval = isInterval;
        this.transComp = transComp;
        isTransport = true;
        initPanel();
        setTimeConstraints();
    }

    private void setTimeConstraints() {
        List<ColoredTimeInvariant> timeInvariants;
        List<ColoredTimeInterval> timeIntervals;

        if (isInterval) {
            if (isTransport)
                timeIntervals = ((ColoredTransportArc)transComp.underlyingTransportArc()).getCtiList();
            else
                timeIntervals = ((ColoredInputArc) arcComp.underlyingTimedInputArc()).getColorTimeIntervals();
            for (ColoredTimeInterval timeInterval : timeIntervals) {
                timeConstraintListModel.addElement(timeInterval);
            }
        } else {
            if (placeComp.underlyingPlace() instanceof LocalColoredPlace) {
                timeInvariants = ((LocalColoredPlace) placeComp.underlyingPlace()).getCtiList();
                for (ColoredTimeInvariant timeInvariant : timeInvariants) {
                    timeConstraintListModel.addElement(timeInvariant);
                }
            }
            else if(placeComp.underlyingPlace()instanceof SharedColoredPlace) {
                timeInvariants = ((SharedColoredPlace) placeComp.underlyingPlace()).getCtiList();
                for (ColoredTimeInvariant timeInvariant : timeInvariants) {
                    timeConstraintListModel.addElement(timeInvariant);
                }
            }
        }

    }

    private void initPanel() {
        initColorConstraintPanel();
    }

    public DefaultListModel getTimeConstraintModel() {return timeConstraintListModel;}

    private MouseListener createDoubleClickMouseAdapter() {
        return new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent arg0) {
                if (!timeConstraintList.isSelectionEmpty()) {
                    if (arg0.getButton() == MouseEvent.BUTTON1 && arg0.getClickCount() == 2) {
                            if (isInterval) {
                                EscapableDialog guiDialog = new EscapableDialog(CreateGui.getApp(), "Edit Time Interval", true);
                                Container contentPane = guiDialog.getContentPane();

                                // 1 Set layout
                                contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));
                                ColoredTimeIntervalDialogPanel ctiPanel = new ColoredTimeIntervalDialogPanel(guiDialog.getRootPane(), context,(ColoredTimeInterval) timeConstraintListModel.getElementAt(timeConstraintList.getSelectedIndex()));
                                contentPane.add(ctiPanel);

                                guiDialog.setResizable(false);

                                // Make window fit contents' preferred size
                                guiDialog.pack();

                                // Move window to the middle of the screen
                                guiDialog.setLocationRelativeTo(null);
                                guiDialog.setVisible(true);

                                if (ctiPanel.isEditConfirmed())
                                    timeConstraintListModel.set(timeConstraintList.getSelectedIndex(), ctiPanel.getNewTimeInterval());

                            } else {
                                EscapableDialog guiDialog = new EscapableDialog(CreateGui.getApp(), "Edit Time Invariant", true);
                                Container contentPane = guiDialog.getContentPane();

                                // 1 Set layout
                                contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));
                                ColoredTimeInvariantDialogPanel ctiPanel = new ColoredTimeInvariantDialogPanel(guiDialog.getRootPane(), context,(ColoredTimeInvariant) timeConstraintListModel.getElementAt(timeConstraintList.getSelectedIndex()), placeComp);
                                contentPane.add(ctiPanel);

                                guiDialog.setResizable(false);

                                // Make window fit contents' preferred size
                                guiDialog.pack();

                                // Move window to the middle of the screen
                                guiDialog.setLocationRelativeTo(null);
                                guiDialog.setVisible(true);

                                if (ctiPanel.didEdit)
                                    timeConstraintListModel.set(timeConstraintList.getSelectedIndex(), ctiPanel.getNewTimeInvariant());
                            }
                        }
                    }
                }
        };
    }

    private void initColorConstraintPanel() {
        nonDefaultColorPanel = new JPanel(new GridBagLayout());
        colorComboboxPanel = new ColorComboboxPanel(colorType, "colors");

        addTimeConstraintButton = new JButton("Add");
        removeTimeConstraintButton = new JButton("Remove");
        editTimeConstraintButton = new JButton("Edit");

        Dimension buttonSize = new Dimension(80, 27);

        addTimeConstraintButton.setPreferredSize(buttonSize);
        removeTimeConstraintButton.setPreferredSize(buttonSize);
        editTimeConstraintButton.setPreferredSize(buttonSize);

        timeConstraintListModel = new DefaultListModel();
        timeConstraintList = new JList(timeConstraintListModel);
        timeConstraintScrollPane = new JScrollPane(timeConstraintList);
        timeConstraintScrollPane.setViewportView(timeConstraintList);
        timeConstraintScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);

        if (isInterval)
            timeConstraintScrollPane.setBorder(BorderFactory.createTitledBorder( "Time interval for colors"));
        else
            timeConstraintScrollPane.setBorder(BorderFactory.createTitledBorder("Time invariant for colors"));
        TableModel tableModel = new DefaultTableModel();
        ((DefaultTableModel) tableModel).addColumn("Color");
        if (isInterval)
            ((DefaultTableModel) tableModel).addColumn("Time interval");
        else
            ((DefaultTableModel) tableModel).addColumn("Time Invariant");
     //   ((DefaultTableModel) tableModel).addRow(test);

        JTable table = new JTable(tableModel);
        table.setPreferredSize(new Dimension(300, 150));
        timeConstraintScrollPane.setPreferredSize(new Dimension(300, 150));
        timeConstraintList.addMouseListener(createDoubleClickMouseAdapter());



        addTimeConstraintButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                JComboBox[] comboBoxes = colorComboboxPanel.getColorTypeComboBoxesArray();
                AbstractTimeConstraint timeConstraint;

                if (!(colorType instanceof ProductType)) {
                    if (isInterval)
                        timeConstraint = ColoredTimeInterval.ZERO_INF_DYN_COLOR((Color) comboBoxes[0].getItemAt(comboBoxes[0].getSelectedIndex()));
                    else
                         timeConstraint = ColoredTimeInvariant.LESS_THAN_INFINITY_DYN_COLOR((Color) comboBoxes[0].getItemAt(comboBoxes[0].getSelectedIndex()));
                } else {
                    Vector<Color> colors = new Vector<Color>();
                    for (int i = 0; i < comboBoxes.length; i++) {
                        colors.add((Color) comboBoxes[i].getItemAt(comboBoxes[i].getSelectedIndex()));
                    }
                    Color color = new Color(colorType, 0, colors);
                    if (isInterval)
                        timeConstraint = ColoredTimeInterval.ZERO_INF_DYN_COLOR(color);
                    else
                        timeConstraint = ColoredTimeInvariant.LESS_THAN_INFINITY_DYN_COLOR(color);
                }
                boolean alreadyExists = false;
                for (int i = 0; i < timeConstraintListModel.size(); i++) {
                    if (timeConstraint.equalsOnlyColor(timeConstraintListModel.get(i)))
                        alreadyExists = true;
                }
                if (alreadyExists) {
                    JOptionPane.showMessageDialog(null, "A time constraint for this color is already active and can be found in the list.");
                } else
                    timeConstraintListModel.addElement(timeConstraint);

            }

        });

        removeTimeConstraintButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                int index = timeConstraintList.getSelectedIndex();
                Object star = timeConstraintListModel.elementAt(index);
                if(star instanceof ColoredTimeInterval) {
                    if(((ColoredTimeInterval) star).getColor().equals(Color.STAR_COLOR)) {
                        JOptionPane.showMessageDialog(null, "Star interval cannot be removed");
                    }else {
                        timeConstraintListModel.removeElementAt(timeConstraintList.getSelectedIndex());
                    }
                }else if(star instanceof ColoredTimeInvariant) {
                    if(((ColoredTimeInvariant) star).getColor().equals(Color.STAR_COLOR)) {
                        JOptionPane.showMessageDialog(null, "Star invariant cannot be removed");
                    }else{
                        timeConstraintListModel.removeElementAt(timeConstraintList.getSelectedIndex());
                    }
                }
            }
        });

        editTimeConstraintButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                if (isInterval) {
                    EscapableDialog guiDialog = new EscapableDialog(CreateGui.getApp(), "Edit Time Interval", true);
                    Container contentPane = guiDialog.getContentPane();

                    // 1 Set layout
                    contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));
                    ColoredTimeIntervalDialogPanel ctiPanel = new ColoredTimeIntervalDialogPanel(guiDialog.getRootPane(), context,(ColoredTimeInterval) timeConstraintListModel.getElementAt(timeConstraintList.getSelectedIndex()));
                    contentPane.add(ctiPanel);

                    guiDialog.setResizable(false);

                    // Make window fit contents' preferred size
                    guiDialog.pack();

                    // Move window to the middle of the screen
                    guiDialog.setLocationRelativeTo(null);
                    guiDialog.setVisible(true);

                    if (ctiPanel.isEditConfirmed())
                        timeConstraintListModel.set(timeConstraintList.getSelectedIndex(), ctiPanel.getNewTimeInterval());

                } else {
                    EscapableDialog guiDialog = new EscapableDialog(CreateGui.getApp(), "Edit Time Invariant", true);
                    Container contentPane = guiDialog.getContentPane();

                    // 1 Set layout
                    contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));
                    ColoredTimeInvariantDialogPanel ctiPanel = new ColoredTimeInvariantDialogPanel(guiDialog.getRootPane(), context,(ColoredTimeInvariant) timeConstraintListModel.getElementAt(timeConstraintList.getSelectedIndex()), placeComp);
                    contentPane.add(ctiPanel);

                    guiDialog.setResizable(false);

                    // Make window fit contents' preferred size
                    guiDialog.pack();

                    // Move window to the middle of the screen
                    guiDialog.setLocationRelativeTo(null);
                    guiDialog.setVisible(true);

                    if (ctiPanel.didEdit)
                        timeConstraintListModel.set(timeConstraintList.getSelectedIndex(), ctiPanel.getNewTimeInvariant());
                }
            }
        });


        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridy = 0;
        gbc.gridx = 0;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.gridwidth = 3;
        nonDefaultColorPanel.add(colorComboboxPanel, gbc);

        gbc = new GridBagConstraints();
        gbc.gridx = 3;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.SOUTHWEST;
        gbc.insets = new Insets(3, 3, 3,3);
        nonDefaultColorPanel.add(addTimeConstraintButton, gbc);

        gbc.gridx = 4;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.SOUTHWEST;
        gbc.insets = new Insets(3, 3, 3, 3);
        nonDefaultColorPanel.add(removeTimeConstraintButton, gbc);

        gbc.gridx = 5;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.SOUTHWEST;
        gbc.insets = new Insets(3, 3, 3, 3);
        nonDefaultColorPanel.add(editTimeConstraintButton, gbc);

        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.gridwidth = 6;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        nonDefaultColorPanel.add(timeConstraintScrollPane, gbc);

        gbc.gridy = 2;
        //nonDefaultColorPanel.add(table, gbc);

        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(3, 3, 3, 3);
        add(nonDefaultColorPanel, gbc);
    }
}
