package pipe.gui.widgets.cpn;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.regex.Pattern;

import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import dk.aau.cs.model.tapn.Colored.ColorType;
import dk.aau.cs.model.tapn.Colored.ProductType;
import org.jdesktop.swingx.JXComboBox;
import pipe.gui.CreateGui;
import dk.aau.cs.model.tapn.TimedArcPetriNetNetwork;
import pipe.gui.widgets.ConstantsPane;
import pipe.gui.widgets.EscapableDialog;
import pipe.gui.widgets.RequestFocusListener;

public class ColorTypeDialogPanel extends javax.swing.JPanel {
    private static final long serialVersionUID = 1L;
    private static final String toolTipColorComboBox = "Switch between the different defined color types";
    private static final String finiteEnumeration= "Finite Enumeration";
    private static final String cyclicEnumeration = "Cyclic Enumeration";
    private static final String rangeOfIntegers = "Range of Integers";
    private static final String productColor = "Product Color";
    private boolean cyclicAndFiniteEnumerationPanelEnabled = false;
    private boolean rangeOfIntegersPanelEnabled = false;
    private boolean productTypePanelEnabled = false;
    private JRootPane rootPane;
    private TimedArcPetriNetNetwork network;
    private EscapableDialog dialog;
    private List<ColorType> colorTypes;
    private ColorType oldColorType;
    private HashMap<String, ColorType> colortypes = new HashMap<String, ColorType>();
    private String oldName;
    private ConstantsPane.ColorTypesListModel colorTypesListModel;

    private JLabel nameLabel;
    private JTextField nameTextField;
    private JComboBox colorTypeComboBox;
    private JLabel colorTypeLabel;
    private JList allColorTypes;
    private JLabel productLabel;
    private DefaultListModel productModel;
    private JButton productAddButton;
    private JButton productRemoveButton;
    private JList productColorTypeList;
    private JComboBox productTypeComboBox;

    private JLabel lowerBoundLabel;
    private JLabel upperBoundLabel;
    private JTextField lowerBoundTextField;
    private JTextField upperBoundTextField;
    private JLabel enumNameLabel;
    private JTextField enumTextField;
    private JButton enumAddButton;
    private JScrollPane enumListScroller;
    private JPanel cyclicAndFiniteEnumerationPanel;
    private JList enumList;
    private JPanel container;
    private JPanel rangeOfIntegersPanel;
    private JPanel productTypePanel;
    private DefaultListModel cyclicModel;
    private JButton cyclicRemoveButton;
    JButton okButton;

    public ColorTypeDialogPanel() throws IOException {
        initComponents();
    }

    public ColorTypeDialogPanel(JRootPane pane, ConstantsPane.ColorTypesListModel colorTypesListModel,
                                TimedArcPetriNetNetwork network) throws IOException {
        rootPane = pane;
        oldName = "";
        this.network = network;
        this.colorTypesListModel = colorTypesListModel;
        initComponents();
        nameTextField.setText(oldName);

    }

    public ColorTypeDialogPanel(JRootPane pane, ConstantsPane.ColorTypesListModel colorTypesListModel,
                                TimedArcPetriNetNetwork network, ColorType colortype) throws IOException {
        rootPane = pane;
        this.oldColorType = colortype;
        oldName = colortype.getName();
        this.network = network;
        this.colorTypesListModel = colorTypesListModel;
        initComponents();
        initValues();
    }

    public void showDialog() {
        dialog = new EscapableDialog(CreateGui.getApp(),
                "Edit color type", true);
        dialog.add(container);
        dialog.getRootPane().setDefaultButton(okButton);
        dialog.setResizable(true);
        dialog.pack();
        dialog.setLocationRelativeTo(null);
        dialog.setVisible(true);
    }

    private void initValues() {
        nameTextField.setText(oldName);
        if (!(oldColorType instanceof ProductType)) { // colortype is always either ProductType or Cyclic
            colorTypeComboBox.setSelectedIndex(2);
            for (dk.aau.cs.model.tapn.Colored.Color element : oldColorType) {
                cyclicModel.addElement(element);
            }
            enumList.setModel(cyclicModel);
        }
        else {
            for (ColorType type : ((ProductType) oldColorType).getColorTypes()) {
                productModel.addElement(type);
            }
            productColorTypeList.setModel(productModel);
        }
    }

    private void initComponents()  {
        container = new JPanel();
        container.setLayout(new GridBagLayout());

        JPanel nameAndTypePanel = createNameAndTypePanel();
        GridBagConstraints gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(5, 8, 0, 8);
        container.add(nameAndTypePanel, gridBagConstraints);

        cyclicAndFiniteEnumerationPanel = createCyclicAndFiniteEnumerationPanel();
        GridBagConstraints gbcFCE = new java.awt.GridBagConstraints();
        gbcFCE.gridx = 0;
        gbcFCE.gridy = 2;
        gbcFCE.anchor = java.awt.GridBagConstraints.WEST;
        gbcFCE.fill = GridBagConstraints.HORIZONTAL;
        gbcFCE.insets = new Insets(5, 8, 0, 8);
        container.add(cyclicAndFiniteEnumerationPanel, gbcFCE);

        rangeOfIntegersPanel = createRangeOfIntegersPanel();
        GridBagConstraints gbcROI = new java.awt.GridBagConstraints();
        gbcROI.gridx = 0;
        gbcROI.gridy = 2;
        gbcROI.anchor = java.awt.GridBagConstraints.WEST;
        gbcROI.fill = GridBagConstraints.HORIZONTAL;
        gbcROI.insets = new Insets(5, 8, 0, 8);
        container.add(rangeOfIntegersPanel, gbcROI);

        productTypePanel = createProductTypePanel();
        GridBagConstraints gbcPT = new java.awt.GridBagConstraints();
        gbcPT.gridx = 0;
        gbcPT.gridy = 2;
        gbcPT.anchor = java.awt.GridBagConstraints.WEST;
        gbcPT.fill = GridBagConstraints.HORIZONTAL;
        gbcPT.insets = new Insets(5, 8, 0, 8);
        container.add(productTypePanel, gbcPT);

        JPanel buttonPanel = createButtonPanel();
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets = new Insets(0, 8, 5, 8);
        gbc.gridx = 0;
        gbc.gridy = 4;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.EAST;
        container.add(buttonPanel,gbc);
        rangeOfIntegersPanel.setVisible(false);
        cyclicAndFiniteEnumerationPanel.setVisible(false);
    }

    private JPanel createNameAndTypePanel() {
        JPanel nameAndTypePanel = new JPanel();
        nameAndTypePanel.setLayout(new GridBagLayout());
        nameAndTypePanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Name and Color Type"));

        nameLabel = new JLabel();
        nameLabel.setText("Name: ");
        GridBagConstraints gbcNL = new GridBagConstraints();
        gbcNL.gridx = 0;
        gbcNL.gridy = 0;
        gbcNL.gridwidth = 1;
        gbcNL.insets = new Insets(4, 4, 2, 4);
        gbcNL.anchor = GridBagConstraints.WEST;
        nameAndTypePanel.add(nameLabel, gbcNL);

        nameTextField = new javax.swing.JTextField();
        Dimension size = new Dimension(330, 30);
        nameTextField.setPreferredSize(size);
        nameTextField.addAncestorListener(new RequestFocusListener());
        nameTextField.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                okButton.requestFocusInWindow();
                okButton.doClick();
            }
        });
        GridBagConstraints gbcNTF = new GridBagConstraints();
        gbcNTF.gridx = 1;
        gbcNTF.gridy = 0;
        gbcNTF.gridwidth = 1;
        gbcNTF.anchor = GridBagConstraints.WEST;
        gbcNTF.fill = GridBagConstraints.HORIZONTAL;
        gbcNTF.insets = new Insets(4, 4, 2, 4);
        nameTextField.requestFocusInWindow();
        nameAndTypePanel.add(nameTextField, gbcNTF);

        colorTypeLabel = new javax.swing.JLabel();
        colorTypeLabel.setText("Color type: ");
        GridBagConstraints gbcCTL = new GridBagConstraints();
        gbcCTL.insets = new Insets(2, 4, 2, 4);
        gbcCTL.gridx = 0;
        gbcCTL.gridy = 1;
        gbcCTL.gridwidth = 1;
        gbcCTL.anchor = GridBagConstraints.WEST;
        nameAndTypePanel.add(colorTypeLabel,gbcCTL);

        colorTypeComboBox = new JXComboBox(new String[]{productColor, cyclicEnumeration, rangeOfIntegers});
        colorTypeComboBox.setToolTipText(toolTipColorComboBox);

        colorTypeComboBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JComboBox source = (JXComboBox)e.getSource();
                    final String selectedString = source.getSelectedItem().toString();

                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                         if (selectedString.equals(finiteEnumeration)) {
                            rangeOfIntegersPanel.setVisible(false);
                            productTypePanel.setVisible(false);
                            cyclicAndFiniteEnumerationPanel.setVisible(true);
                            cyclicAndFiniteEnumerationPanelEnabled = true;
                            rangeOfIntegersPanelEnabled = false;
                            productTypePanelEnabled = false;
                        }
                        else if (selectedString.equals(cyclicEnumeration)) {
                            rangeOfIntegersPanel.setVisible(false);
                            productTypePanel.setVisible(false);
                            cyclicAndFiniteEnumerationPanel.setVisible(true);
                            cyclicAndFiniteEnumerationPanelEnabled = true;
                            rangeOfIntegersPanelEnabled = false;
                            productTypePanelEnabled = false;

                        }
                        else if (selectedString.equals(rangeOfIntegers)) {
                            cyclicAndFiniteEnumerationPanel.setVisible(false);
                            productTypePanel.setVisible(false);
                            rangeOfIntegersPanel.setVisible(true);
                            rangeOfIntegersPanelEnabled = true;
                            cyclicAndFiniteEnumerationPanelEnabled = false;
                            productTypePanelEnabled = false;

                        }
                        else if (selectedString.equals(productColor)) {
                            cyclicAndFiniteEnumerationPanel.setVisible(false);
                            productTypePanel.setVisible(true);
                            rangeOfIntegersPanel.setVisible(false);
                            productTypePanelEnabled = true;
                            rangeOfIntegersPanelEnabled = false;
                            cyclicAndFiniteEnumerationPanelEnabled = false;
                        }
                    }
                });
            }
        });
        GridBagConstraints gbcCTCB = new GridBagConstraints();
        gbcCTCB.insets = new Insets(2, 4, 2, 4);
        Dimension conboBoxSize = new Dimension(350,30);
        colorTypeComboBox.setPreferredSize(conboBoxSize);
        colorTypeComboBox.setMaximumSize(conboBoxSize);
        colorTypeComboBox.setMinimumSize(conboBoxSize);
        colorTypeComboBox.setBackground(Color.white);
        gbcCTCB.gridx = 1;
        gbcCTCB.gridy = 1;
        gbcCTCB.gridwidth = 1;
        gbcCTCB.anchor = GridBagConstraints.EAST;
        gbcCTCB.fill = GridBagConstraints.HORIZONTAL;
        nameAndTypePanel.add(colorTypeComboBox, gbcCTCB);

        return nameAndTypePanel;
    }

    private JPanel createButtonPanel() {
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridBagLayout());

        okButton = new JButton("OK");
        okButton.setMaximumSize(new java.awt.Dimension(100, 25));
        okButton.setMinimumSize(new java.awt.Dimension(100, 25));
        okButton.setPreferredSize(new java.awt.Dimension(100, 25));

        okButton.setMnemonic(KeyEvent.VK_O);
        GridBagConstraints gbcOk = new GridBagConstraints();
        gbcOk.gridx = 1;
        gbcOk.gridy = 0;
        gbcOk.anchor = java.awt.GridBagConstraints.WEST;
        gbcOk.insets = new java.awt.Insets(5, 5, 5, 5);

        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent actionEvent) {
                onOK();
            }
        });

        JButton cancelButton = new JButton("Cancel");
        cancelButton.setMaximumSize(new java.awt.Dimension(100, 25));
        cancelButton.setMinimumSize(new java.awt.Dimension(100, 25));
        cancelButton.setPreferredSize(new java.awt.Dimension(100, 25));
        cancelButton.setMnemonic(KeyEvent.VK_C);

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = java.awt.GridBagConstraints.RELATIVE;
        gbc.anchor = GridBagConstraints.EAST;

        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                exit();
            }
        });

        buttonPanel.add(cancelButton,gbc);
        buttonPanel.add(okButton,gbcOk);

        return buttonPanel;
    }

    private JPanel createRangeOfIntegersPanel() {
        JPanel rangeOfIntegers = new JPanel();
        rangeOfIntegers.setLayout(new GridBagLayout());
        rangeOfIntegers.setBorder(javax.swing.BorderFactory.createTitledBorder("Range Of Integers"));

        lowerBoundLabel = new JLabel("Lower Bound: ");
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.insets = new Insets(3, 3, 3, 3);
        rangeOfIntegers.add(lowerBoundLabel, gbc);

        lowerBoundTextField = new  JTextField();

        Dimension size = new Dimension(55, 30);
        lowerBoundTextField.setPreferredSize(size);
        lowerBoundTextField.setMaximumSize(size);
        lowerBoundTextField.setMaximumSize(size);

        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(4, 4, 2, 4);
        rangeOfIntegers.add(lowerBoundTextField, gbc);

        upperBoundLabel = new JLabel("Upper Bound: ");
        gbc.gridx = 3;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.insets = new Insets(3, 3, 3, 3);
        rangeOfIntegers.add(upperBoundLabel, gbc);

        upperBoundTextField = new  JTextField();
        upperBoundTextField.setPreferredSize(size);
        upperBoundTextField.setMaximumSize(size);
        upperBoundTextField.setMaximumSize(size);

        gbc.gridx = 4;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(4, 4, 2, 4);
        //lowerBoundTextField.requestFocusInWindow();
        rangeOfIntegers.add(upperBoundTextField, gbc);

        return rangeOfIntegers;
    }

    private JPanel createCyclicAndFiniteEnumerationPanel() {
        final JPanel cyclicAndFiniteEnumeration = new JPanel();
        cyclicAndFiniteEnumeration.setLayout(new GridBagLayout());
        cyclicAndFiniteEnumeration.setBorder(javax.swing.BorderFactory.createTitledBorder("Cyclic and Finite Enumeration"));

        JPanel firstRow = new JPanel();
        firstRow.setLayout(new GridBagLayout());

        enumNameLabel = new JLabel("Name: ");
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.insets = new Insets(3, 3, 3, 3);
        firstRow.add(enumNameLabel, gbc);

        enumTextField = new  JTextField();
        Dimension size = new Dimension(380, 30);
        enumTextField.setPreferredSize(size);
        enumTextField.setMaximumSize(size);
        enumTextField.setMaximumSize(size);
        gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.gridwidth = 3;
        gbc.insets = new Insets(4, 4, 2, 4);
        firstRow.add(enumTextField, gbc);


        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        cyclicAndFiniteEnumeration.add(firstRow, gbc);

        JPanel secondRow = new JPanel();
        secondRow.setLayout(new GridBagLayout());

        enumAddButton = new JButton("Add");
        enumAddButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String enumerationName = enumTextField.getText();
                if(enumerationName == null || enumerationName.trim().isEmpty()) {
                    JOptionPane.showMessageDialog(
                            CreateGui.getApp(), "You have to enter a name for the color",
                            "Error", JOptionPane.ERROR_MESSAGE);
                    return;
                }else {
                    cyclicModel.addElement(enumTextField.getText());
                    enumList.setModel(cyclicModel);
                    enumTextField.setText("");
                    cyclicRemoveButton.setEnabled(true);
                }
            }
        });

        Dimension buttonSize = new Dimension(100, 30);
        enumAddButton.setMaximumSize(buttonSize);
        enumAddButton.setMinimumSize(buttonSize);
        enumAddButton.setPreferredSize(buttonSize);
        enumAddButton.setMnemonic(KeyEvent.VK_A);

        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        secondRow.add(enumAddButton, gbc);

        cyclicRemoveButton = new JButton("Remove");
        cyclicRemoveButton.setEnabled(false);
        cyclicRemoveButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                cyclicModel.removeElementAt(enumList.getSelectedIndex());
                enumList.setModel(cyclicModel);
            }
        });
        cyclicRemoveButton.setPreferredSize(buttonSize);
        cyclicRemoveButton.setMinimumSize(buttonSize);
        cyclicRemoveButton.setMaximumSize(buttonSize);

        gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.WEST;
        secondRow.add(cyclicRemoveButton, gbc);

        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        cyclicAndFiniteEnumeration.add(secondRow, gbc);

        JPanel thirdRow = new JPanel();
        thirdRow.setLayout(new GridBagLayout());

        cyclicModel = new DefaultListModel();
        enumList = new JList();

        enumList.addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting()) {
                    JList source = (JList) e.getSource();
                    if (source.getSelectedIndex() == -1) {
                        cyclicRemoveButton.setEnabled(false);
                    }
                }
            }
        });

        JScrollPane cyclicListScrollPane = new JScrollPane(enumList);
        cyclicListScrollPane.setViewportView(enumList);
        cyclicListScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);


        gbc.gridx = 0;
        gbc.gridy = 0;
        Dimension listSize = new Dimension(450, 150);
        cyclicListScrollPane.setPreferredSize(listSize);
        cyclicListScrollPane.setMinimumSize(listSize);
        cyclicListScrollPane.setMaximumSize(listSize);
        cyclicListScrollPane.setVisible(true);
        cyclicListScrollPane.setBorder(new LineBorder(Color.GRAY));
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        thirdRow.add(cyclicListScrollPane, gbc);

        gbc = new GridBagConstraints();
        gbc.gridy = 2;
        gbc.gridx = 0;
        cyclicAndFiniteEnumeration.add(thirdRow, gbc);

        return cyclicAndFiniteEnumeration;
    }

    private JPanel createProductTypePanel() {
        JPanel productTypePanel = new JPanel();
        productTypePanelEnabled = true;
        productTypePanel.setLayout(new GridBagLayout());
        productTypePanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Product Type/ Domain"));
        GridBagConstraints gbc = new GridBagConstraints();

        productLabel = new JLabel("Color types: ");
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.insets = new Insets(3, 3, 3, 3);
        productTypePanel.add(productLabel, gbc);

        productModel = new DefaultListModel();

        allColorTypes = new JList();


        colorTypes = new ArrayList<ColorType>();
        colorTypes = network.colorTypes();

        productTypeComboBox = new javax.swing.JComboBox();

        for (ColorType element : colorTypes) {
            productTypeComboBox.addItem(element.getName());
        }
        gbc.insets = new Insets(2, 4, 2, 4);
        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        Dimension ComboBoxSize = new Dimension(350,30);
        productTypeComboBox.setPreferredSize(ComboBoxSize);
        productTypeComboBox.setBackground(Color.white);
        gbc.anchor = GridBagConstraints.EAST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        productTypePanel.add(productTypeComboBox,gbc);

        JPanel productButtonPanel = createProductButtonsPanel();
        gbc.gridx = 1;
        gbc.gridy = 2;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.insets = new Insets(3, 3, 3, 3);
        productTypePanel.add(productButtonPanel, gbc);

        productColorTypeList = new JList();
        JScrollPane productColorsListScrollPane = new JScrollPane(productColorTypeList);
        productColorsListScrollPane.setViewportView(productColorTypeList);
        productColorsListScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
        Dimension size = new Dimension(300,150);
        productColorsListScrollPane.setPreferredSize(size);
        productColorsListScrollPane.setBorder(new LineBorder(Color.GRAY));
        gbc.gridx = 1;
        gbc.gridy = 3;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.insets = new Insets(3,3,3,3);
        productTypePanel.add(productColorsListScrollPane, gbc);

        return productTypePanel;
    }

    private JPanel createProductButtonsPanel() {
        JPanel productButtonPanel = new JPanel();
        productButtonPanel.setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();

        productAddButton = new JButton("Add");
        productAddButton.setMnemonic(KeyEvent.VK_A);
        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.gridwidth = java.awt.GridBagConstraints.RELATIVE;
        gbc.anchor = GridBagConstraints.EAST;
        productButtonPanel.add(productAddButton, gbc);

        productAddButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                productModel.addElement(productTypeComboBox.getSelectedItem());
                productColorTypeList.setModel(productModel);
                productRemoveButton.setEnabled(true);
            }
        });

        productRemoveButton = new JButton("Remove");
        productRemoveButton.setEnabled(false);
        productRemoveButton.setMnemonic(KeyEvent.VK_R);
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = java.awt.GridBagConstraints.RELATIVE;
        gbc.anchor = GridBagConstraints.EAST;
        productButtonPanel.add(productRemoveButton, gbc);

        productRemoveButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {

                productModel.removeElementAt(productColorTypeList.getSelectedIndex());

            }
        });
        //productColorTypeList.addListSelectionListener(new ListSelectionListener() {
        //    @Override
        //    public void valueChanged(ListSelectionEvent e) {
        //        if (!e.getValueIsAdjusting()) {
        //            JList source = (JList) e.getSource();
        //            if (source.getSelectedIndex() == -1) {
        //                productRemoveButton.setEnabled(false);
        //            }
        //        }
        //    }
        //});

        return productButtonPanel;
    }

    private void exit() {
        dialog.setVisible(false);
    }

    private void onOK() {
            String name = nameTextField.getText();
            String lowerbound = lowerBoundTextField.getText();
            String upperbound = upperBoundTextField.getText();
            String enumerationName = enumTextField.getText();


            if (name == null || name.trim().isEmpty()) {
                JOptionPane.showMessageDialog(
                        CreateGui.getApp(), "You have to enter a name for the color",
                        "Error", JOptionPane.ERROR_MESSAGE);
                return;
            }else if (!Pattern.matches("[a-zA-Z]([\\_a-zA-Z0-9])*", name)) {
                JOptionPane.showMessageDialog(
                                CreateGui.getApp(),
                                "Acceptable names for color types are defined by the regular expression:\n[a-zA-Z][_a-zA-Z0-9]*",
                                "Error", JOptionPane.ERROR_MESSAGE);
                return;
            } else if(network.isNameUsedForColorType(name) && oldColorType == null) {
                JOptionPane.showMessageDialog(
                                CreateGui.getApp(),
                                "There is already another color type with the same name.\n\n"
                                        + "Choose a different name for the color type.",
                                "Error", JOptionPane.ERROR_MESSAGE);
                return;
            }else if (!oldName.equals("") && !oldName.equalsIgnoreCase(name) && network.isNameUsedForColorType(name)) {
                JOptionPane.showMessageDialog(
                                CreateGui.getApp(),
                                "There is already another color type with the same name.\n\n"
                                        + "Choose a different name for the color type.",
                                "Error", JOptionPane.ERROR_MESSAGE);
                return;
            }else if (!Pattern.matches("[0-9]+", lowerbound) && rangeOfIntegersPanelEnabled ) {
                JOptionPane.showMessageDialog(
                                CreateGui.getApp(),
                                "Lower bound must be a number",
                                "Error", JOptionPane.ERROR_MESSAGE);
                return;
            } else if (!Pattern.matches("[0-9]+", upperbound) && rangeOfIntegersPanelEnabled ) {
                JOptionPane.showMessageDialog(
                                CreateGui.getApp(),
                                "Upper bound must be a number",
                                "Error", JOptionPane.ERROR_MESSAGE);
                return;
            }
            else if (rangeOfIntegersPanelEnabled && (Integer.parseInt(lowerBoundTextField.getText()) > Integer.parseInt(upperBoundTextField.getText()))) {
                JOptionPane.showMessageDialog(
                        CreateGui.getApp(),
                        "Lower bound must be smaller than Upper bound",
                        "Error", JOptionPane.ERROR_MESSAGE);
                return;
            }
            else if((lowerbound.trim().isEmpty() || upperbound.trim().isEmpty()) && rangeOfIntegersPanelEnabled) {
                JOptionPane.showMessageDialog(
                        CreateGui.getApp(),
                        "You must specify both a lower and upper bound",
                        "Error", JOptionPane.ERROR_MESSAGE);
                return;
            }

            else {
                String selectedColorType = colorTypeComboBox.getSelectedItem().toString();
                ColorType newColorType = new ColorType(name);

                 if(selectedColorType.equals(finiteEnumeration) || selectedColorType.equals(cyclicEnumeration)) {
                    for(int i = 0; i < enumList.getModel().getSize();i++) {
                        newColorType.addColor(enumList.getModel().getElementAt(i).toString());
                    }
                    if (oldColorType != null) {
                        network.updateColorType(oldColorType.getName(), newColorType);
                        colorTypesListModel.updateName();
                    } else {
                        colorTypesListModel.addElement(newColorType);
                    }

                }
                else if(selectedColorType.equals(rangeOfIntegers)) {
                    Integer lowerboundNumber = Integer.parseInt(lowerbound);
                    Integer upperboundNumber = Integer.parseInt(upperbound);
                    for(int i = lowerboundNumber; i < upperboundNumber + 1; i++) {
                        newColorType.addColor(String.valueOf(i));
                    }
                    if (oldColorType != null) {
                        network.updateColorType(oldColorType.getName(), newColorType);
                        colorTypesListModel.updateName();
                    }
                    else
                        colorTypesListModel.addElement(newColorType);
                }
                else if(selectedColorType.equals(productColor)) {
                    ProductType productType = new ProductType(name);
                    int size = productColorTypeList.getModel().getSize();
                    for(int i = 0; i < size; i++) {
                        String colorTypeName = productColorTypeList.getModel().getElementAt(i).toString();
                        int size2 = productTypeComboBox.getItemCount();
                        for (int j = 0; j < size2; j++) {
                            if (colorTypeName.equals(productTypeComboBox.getItemAt(j).toString())) {
                                productType.addType(colorTypes.get(j));
                            }
                        }
                    }
                    if (oldColorType != null) {
                        network.updateColorType(oldColorType.getName(), productType);
                        colorTypesListModel.updateName();
                    } else {
                        colorTypesListModel.addElement(productType);
                    }
                }
            }
            exit();
    }


}
