package pipe.gui.widgets.cpn;

import dk.aau.cs.debug.Logger;
import dk.aau.cs.gui.Context;
import dk.aau.cs.model.tapn.Colored.*;
import dk.aau.cs.model.tapn.Colored.Color;
import dk.aau.cs.model.tapn.Colored.ExpressionSupport.ExprStringPosition;
import dk.aau.cs.model.tapn.Colored.Expressions.*;
import pipe.gui.CreateGui;
import pipe.gui.graphicElements.Arc;
import pipe.gui.graphicElements.PetriNetObject;
import pipe.gui.graphicElements.Place;
import pipe.gui.graphicElements.cpn.ColoredInputArcComponent;
import pipe.gui.graphicElements.cpn.ColoredOutputArcComponent;
import pipe.gui.graphicElements.cpn.ColoredTransportArcComponent;
import pipe.gui.widgets.EscapableDialog;
import pipe.gui.widgets.GuardDialogue;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

public class ColoredArcDialogPanel extends GuardDialogue {

    private JPanel exprPanel;
    private JTextPane exprField;
    private JScrollPane exprScrollPane;

    private JPanel numberExprPanel;
    private JSpinner numberExpressionJSpinner;
    private JButton numberExpressionButton;
    private JComboBox<ColorType> allExpressionComboBox;
    private JSpinner allExpressionJSpinner;
    private JButton allExpressionButton;

    private JPanel arithmeticPanel;
    private JButton additionButton;
    private JButton addAdditionPlaceHolderButton;
    private JButton subtractionButton;
    private JButton scalarButton;
    private JButton addColorExpressionButton;
    private JSpinner scalarJSpinner;

    private ColorIntervalAndInvariantEditPanel colorIntervalPanel;

    private JPanel editPanel;
    private ButtonGroup editButtonsGroup;
    private JButton resetExprButton;
    private JButton deleteExprSelectionButton;
    private JButton editExprButton;
    private JButton undoButton;
    private JButton redoButton;

    private ColoredInputArcComponent objectToBeEditedInput;
    private ColoredOutputArcComponent objectToBeEditedOutput;
    private ColoredTransportArcComponent objectToBeEditedTransport;
    private boolean input;
    private boolean transportArc;
    private Context context;
    private ArcExpression arcExpression;
    private ExprStringPosition currentSelection = null;
    private PetriNetObject petriNetObject;
    private ColoredTimeInterval activeTimeGuard;

    public ColoredArcDialogPanel(JRootPane rootPane, PetriNetObject objectToBeEdited, Context context) {
        super(rootPane, objectToBeEdited);
        this.context = context;

        if (objectToBeEdited instanceof ColoredTransportArcComponent) {
            if (((ColoredTransportArcComponent) objectToBeEdited).getSource() instanceof Place) {
                this.objectToBeEditedTransport = (ColoredTransportArcComponent)objectToBeEdited;
                input = true;
                transportArc = true;
            }
            else {
                this.objectToBeEditedTransport = (ColoredTransportArcComponent)objectToBeEdited;
                input = false;
                transportArc = true;
            }
        }
        else if (objectToBeEdited instanceof ColoredInputArcComponent) {
            this.objectToBeEditedInput = (ColoredInputArcComponent) objectToBeEdited;
            input = true;
            transportArc = false;
        } else if (objectToBeEdited instanceof ColoredOutputArcComponent) {
            this.objectToBeEditedOutput = (ColoredOutputArcComponent) objectToBeEdited;
            transportArc = false;
            input = false;
        }
        petriNetObject = objectToBeEdited;
        initComponents();
        undoButton.setEnabled(false);
        redoButton.setEnabled(false);
        editExprButton.setEnabled(false);
    }

    private void initComponents() {
        initPanels();
        initExpr();
        initonOK();
    }




    private void initExpr() {
        if (input != true) {
            if (((ColoredOutputArc) objectToBeEditedOutput.underlyingArc()).getExpression() != null) {
                PlaceHolderArcExpression placeholderArc = new PlaceHolderArcExpression();
                arcExpression = placeholderArc;
                updateSelection(arcExpression);
                parseExpression(((ColoredOutputArc) objectToBeEditedOutput.underlyingArc()).getExpression());
            } else {
                arcExpression = new PlaceHolderArcExpression();
                exprField.setText(arcExpression.toString());
            }
        }
        else {
            if (transportArc != true) {
                ColoredInputArc inputArc = (ColoredInputArc) objectToBeEditedInput.underlyingTimedInputArc();
                if (inputArc.getArcExpression() != null) {
                    PlaceHolderArcExpression placeholderArc = new PlaceHolderArcExpression();
                    arcExpression = placeholderArc;
                    updateSelection(arcExpression);
                    parseExpression(((ColoredInputArc) objectToBeEditedInput.underlyingTimedInputArc()).getArcExpression());
                } else {
                    arcExpression = new PlaceHolderArcExpression();
                    exprField.setText(arcExpression.toString());
                }
            } else {
                ColoredTransportArc transportArc = (ColoredTransportArc) objectToBeEditedTransport.underlyingTransportArc();
                if (transportArc.getExpression() != null) {
                    PlaceHolderArcExpression placeHolderArcExpression = new PlaceHolderArcExpression();
                    arcExpression = placeHolderArcExpression;
                    updateSelection(arcExpression);
                    parseExpression(((ColoredTransportArc) objectToBeEditedTransport.underlyingTransportArc()).getExpression());
                } else {
                    arcExpression = new PlaceHolderArcExpression();
                    exprField.setText(arcExpression.toString());
                }
            }

        }
    }

    private void toggleColorExprButton(boolean enable) {
        addColorExpressionButton.setEnabled(enable);
    }

    private void toggleArcExpressionButtons(boolean enable) {
        allExpressionButton.setEnabled(enable);
        numberExpressionButton.setEnabled(enable);
        additionButton.setEnabled(enable);
        subtractionButton.setEnabled(enable);
        addAdditionPlaceHolderButton.setEnabled(enable);
        scalarButton.setEnabled(enable);
    }

    private void toggleaddPlaceHolderButton(boolean enable) {
        addAdditionPlaceHolderButton.setEnabled(enable);
    }

    private void toggleEnabledButtons() {
        if (currentSelection.getObject() instanceof ColorExpression) {
            toggleaddPlaceHolderButton(false);
            toggleColorExprButton(true);
        }
        else if (currentSelection.getObject() instanceof AddExpression) {
            toggleArcExpressionButtons(false);
            toggleColorExprButton(false);
            toggleaddPlaceHolderButton(true);
        }
        else if (currentSelection.getObject() instanceof ArcExpression) {
            toggleArcExpressionButtons(true);
            toggleColorExprButton(false);
            toggleaddPlaceHolderButton(false);
        }

    }

    private void updateSelection() {
        int index = exprField.getCaretPosition();
        ExprStringPosition position = arcExpression.objectAt(index);
        Logger.log(position.getObject().toString());
        if (position == null) {
            return;
        }

        exprField.select(position.getStart(), position.getEnd());
        currentSelection = position;

        if (currentSelection != null) {
            toggleEnabledButtons();
        } {
            //TODO: disable all expr buttons
        }

        //TODO: updateexprButtonsAccordingToSelection; line 573
    }

    private void updateSelection(Expression newSelection) {
        exprField.setText(arcExpression.toString());

        ExprStringPosition position;
        if (arcExpression.containsPlaceHolder()) {
            Expression ae = arcExpression.findFirstPlaceHolder();
            position = arcExpression.indexOf(ae);
        }
         else {
             position = arcExpression.indexOf(newSelection);
        }

        exprField.select(position.getStart(), position.getEnd());
         currentSelection = position;

         if (currentSelection != null) {
             toggleEnabledButtons();
         }
         else {
             //TODO::
         }
    }

    private void deleteSelection() {
        if (currentSelection != null) {
            Expression replacement = null;
            if (currentSelection.getObject() instanceof ArcExpression) {
                replacement = getSpecificChildOfProperty(1, currentSelection.getObject());
            }
            else if (currentSelection.getObject() instanceof ArcExpression) {
                replacement = new PlaceHolderColorExpression();
            }
            if (replacement != null) {
                arcExpression = arcExpression.replace(currentSelection.getObject(), replacement);
                updateSelection(replacement);
            }
        }
    }

    private ArcExpression getSpecificChildOfProperty(int number, Expression property) {
        ExprStringPosition[] children = property.getChildren();
        int count = 0;
        for (int i = 0; i < children.length; i++) {
            Expression child = children[i].getObject();
            if (child instanceof ArcExpression) {
                count++;
            }
            if (count == number) {
                return (ArcExpression) child;
            }
        }

        return new PlaceHolderArcExpression();
    }

    private void addAllExpression() {
        AllExpression allExpr = new AllExpression(allExpressionComboBox.getItemAt(allExpressionComboBox.getSelectedIndex()));
        Integer value = (Integer)allExpressionJSpinner.getValue();
        NumberOfExpression numbExpr = new NumberOfExpression(value, allExpr);
        if (currentSelection.getObject() instanceof ArcExpression) {
            arcExpression = arcExpression.replace(currentSelection.getObject(), numbExpr);
            updateSelection(numbExpr);
        }
    }

    private void addNumberExpression() {
        Vector<ColorExpression> colorExprVec = new Vector();
        colorExprVec.add(new PlaceHolderColorExpression());
        Integer value = (Integer)numberExpressionJSpinner.getValue();
        NumberOfExpression numbExpr = new NumberOfExpression(value, colorExprVec);
        arcExpression = arcExpression.replace(currentSelection.getObject(), numbExpr);
        updateSelection(numbExpr);
    }

    private void parseExpression(ArcExpression expressionToParse) {
        if (expressionToParse instanceof AddExpression) {
            AddExpression addExpr = new AddExpression(((AddExpression) expressionToParse).getAddExpression());
            arcExpression = arcExpression.replace(currentSelection.getObject(), addExpr);
            updateSelection(addExpr);
        }
        else if(expressionToParse instanceof SubtractExpression) {
            SubtractExpression subExpr = new SubtractExpression(((SubtractExpression) expressionToParse).getLeftExpression(), ((SubtractExpression) expressionToParse).getRightExpression());
            arcExpression = arcExpression.replace(currentSelection.getObject(), subExpr);
            updateSelection(subExpr);
        }
        else if (expressionToParse instanceof NumberOfExpression) {
            NumberOfExpression expessionToParse = (NumberOfExpression) expressionToParse;
            NumberOfExpression numbExpr = null;
            if (expessionToParse.getAll() != null) {
                 numbExpr = new NumberOfExpression(((NumberOfExpression) expressionToParse).getNumber(), expessionToParse.getAll());
            } else {
                 numbExpr = new NumberOfExpression(expessionToParse.getNumber(), expessionToParse.getColor());
            }
            arcExpression = arcExpression.replace(currentSelection.getObject(), numbExpr);
            updateSelection(numbExpr);
        }
    }

    private void initonOK() {
        okButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                onOkColored();
            }
        });
    }

    private void onOkColored() {
        if (input) {
            if (transportArc) {
                ColoredTransportArc transportArc = (ColoredTransportArc)objectToBeEditedTransport.underlyingTransportArc();
                transportArc.setExpression(arcExpression);
                objectToBeEditedTransport.setUnderlyingArc(transportArc);
                ((ColoredTransportArc)((ColoredTransportArcComponent)petriNetObject).underlyingTransportArc()).setExpression(arcExpression);
                Logger.log(getctiList().size());
                ((ColoredTransportArc)((ColoredTransportArcComponent)petriNetObject).underlyingTransportArc()).setCtiList(getctiList());
                objectToBeEditedTransport.updateLabel(false);
            } else {
                ColoredInputArc inputArc = (ColoredInputArc)objectToBeEditedInput.underlyingTimedInputArc();
                inputArc.setExpression(arcExpression);
                objectToBeEditedInput.setUnderlyingArc(inputArc);
                ((ColoredInputArc)((ColoredInputArcComponent)petriNetObject).underlyingTimedInputArc()).setExpression(arcExpression);
                Logger.log(getctiList().size());
                ((ColoredInputArc)((ColoredInputArcComponent)petriNetObject).underlyingTimedInputArc()).setColorTimeIntervals(getctiList());
                objectToBeEditedInput.updateLabel(false);
            }
        } else {
            ColoredOutputArc outputArc = (ColoredOutputArc)objectToBeEditedOutput.underlyingArc();
            objectToBeEditedOutput.setUnderlyingArc(outputArc);
            ((ColoredOutputArc)((ColoredOutputArcComponent)petriNetObject).underlyingArc()).setExpression(arcExpression);
        }

    }

    private List<ColoredTimeInterval> getctiList() {
        List<ColoredTimeInterval> ctiList = new ArrayList<ColoredTimeInterval>();
        for (int i = 0; i < colorIntervalPanel.getTimeConstraintModel().size(); i++) {
            ctiList.add((ColoredTimeInterval) colorIntervalPanel.getTimeConstraintModel().get(i));
        }
        return ctiList;
    }

    private void initPanels() {
        exprPanel = new JPanel(new GridBagLayout());
        exprPanel.setBorder(BorderFactory.createTitledBorder("Arc Expressions"));
        if (input)
            initColoredTimedGuard();

        initExprField();
        initNumberExpressionsPanel();
        initArithmeticPanel();
        initEditPanel();

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 2;
        gbc.insets = new Insets(5, 10, 5, 10);
        gbc.fill = GridBagConstraints.BOTH;
        gbc.anchor = GridBagConstraints.WEST;
        add(exprPanel, gbc);
    }


    private void initEditPanel() {
        editPanel = new JPanel(new GridBagLayout());
        editPanel.setBorder(BorderFactory.createTitledBorder("Editing"));
        editPanel.setPreferredSize(new Dimension(260, 190));

        ButtonGroup editButtonsGroup = new ButtonGroup();
        deleteExprSelectionButton = new JButton("Delete Selection");
        resetExprButton = new JButton("Reset Expression");
        undoButton = new JButton("Undo");
        redoButton = new JButton("Redo");
        editExprButton = new JButton("Edit Expression");
        editExprButton.setEnabled(true);

        //TODO: add tooltips to buttons

        editButtonsGroup.add(deleteExprSelectionButton);
        editButtonsGroup.add(resetExprButton);
        editButtonsGroup.add(undoButton);
        editButtonsGroup.add(redoButton);
        editButtonsGroup.add(editExprButton);

        deleteExprSelectionButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                deleteSelection();
            }
        });

        resetExprButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                PlaceHolderArcExpression pHExpr = new PlaceHolderArcExpression();
                arcExpression = arcExpression.replace(arcExpression, pHExpr);
                updateSelection(pHExpr);
            }
        });

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets = new Insets(5,5,5,5);
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.WEST;
        editPanel.add(undoButton, gbc);

        gbc.gridx = 1;
        gbc.insets = new Insets(0, 10, 0 , 0);
        editPanel.add(redoButton, gbc);

        gbc.insets = new Insets(0, 0, 5 , 0);
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.gridwidth = 2;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        editPanel.add(deleteExprSelectionButton, gbc);

        gbc.gridy = 2;
        editPanel.add(resetExprButton, gbc);

        gbc.gridy = 3;
        editPanel.add(editExprButton, gbc);

        //TODO: Actionlisteners

        gbc = new GridBagConstraints();
        gbc.gridx = 2;
        gbc.gridy = 1;
        gbc.fill = GridBagConstraints.VERTICAL;
        exprPanel.add(editPanel, gbc);
    }

    private void initArithmeticPanel() {
        arithmeticPanel = new JPanel(new GridBagLayout());
        arithmeticPanel.setBorder(BorderFactory.createTitledBorder("Arithmetic Expressions"));

        additionButton = new JButton("Addition");
        addAdditionPlaceHolderButton = new JButton("Add Placeholder");
        subtractionButton = new JButton("Subtraction");
        scalarButton = new JButton("Scalar");

        final Integer current = 1;
        Integer min = 1;
        Integer max = 999;
        Integer step = 1;
        SpinnerNumberModel numberModelScalar = new SpinnerNumberModel(current, min, max, step);

        scalarJSpinner = new JSpinner(numberModelScalar);

        scalarJSpinner.setPreferredSize(new Dimension(50, 27));
        scalarJSpinner.setPreferredSize(new Dimension(50, 27));
        scalarJSpinner.setPreferredSize(new Dimension(50, 27));

        additionButton.setPreferredSize(new Dimension(110, 30));
        additionButton.setMinimumSize(new Dimension(110, 30));
        additionButton.setMaximumSize(new Dimension(110, 30));

        subtractionButton.setPreferredSize(new Dimension(110, 30));
        subtractionButton.setMinimumSize(new Dimension(110, 30));
        subtractionButton.setMaximumSize(new Dimension(110, 30));

        scalarButton.setPreferredSize(new Dimension(110, 30));
        scalarButton.setMinimumSize(new Dimension(110, 30));
        scalarButton.setMaximumSize(new Dimension(110, 30));

        addAdditionPlaceHolderButton.setPreferredSize(new Dimension(150, 30));

        additionButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                AddExpression addExpr;
                if (currentSelection.getObject() instanceof ArcExpression) {
                    Vector<ArcExpression> vExpr = new Vector();
                    vExpr.add((ArcExpression) currentSelection.getObject());
                    vExpr.add(new PlaceHolderArcExpression());
                    addExpr = new AddExpression(vExpr);
                    arcExpression = arcExpression.replace(currentSelection.getObject(), addExpr);
                    updateSelection(addExpr);
                }
            }
        });

        subtractionButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                SubtractExpression subExpr = null;
                if (currentSelection.getObject() instanceof PlaceHolderArcExpression) {
                    subExpr = new SubtractExpression((PlaceHolderArcExpression)currentSelection.getObject(), new PlaceHolderArcExpression());
                    arcExpression = arcExpression.replace(currentSelection.getObject(), subExpr);
                    updateSelection(subExpr);
                }
                else if (currentSelection.getObject() instanceof SubtractExpression) {
                    subExpr = new SubtractExpression((SubtractExpression)currentSelection.getObject(), new PlaceHolderArcExpression());
                    arcExpression = arcExpression.replace(currentSelection.getObject(), subExpr);
                    updateSelection(subExpr);
                }
                else if (currentSelection.getObject() instanceof ScalarProductExpression) {
                    subExpr = new SubtractExpression((ScalarProductExpression)currentSelection.getObject(), new PlaceHolderArcExpression());
                    arcExpression = arcExpression.replace(currentSelection.getObject(), subExpr);
                    updateSelection(subExpr);
                } else if (currentSelection.getObject() instanceof NumberOfExpression || currentSelection.getObject() instanceof AddExpression) {
                    subExpr = new SubtractExpression((ArcExpression) currentSelection.getObject(), new PlaceHolderArcExpression());
                    arcExpression = arcExpression.replace(currentSelection.getObject(), subExpr);
                    updateSelection(subExpr);
                }
            }
        });

        scalarButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                ScalarProductExpression scalarExpr = null;
                Integer value = (Integer)scalarJSpinner.getValue();
                if (currentSelection.getObject() instanceof PlaceHolderArcExpression) {
                    scalarExpr = new ScalarProductExpression(value, (PlaceHolderArcExpression)currentSelection.getObject());
                    arcExpression = arcExpression.replace(currentSelection.getObject(), scalarExpr);
                    updateSelection(scalarExpr);
                }
            }
        });

        addAdditionPlaceHolderButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                if (currentSelection.getObject() instanceof AddExpression) {
                    AddExpression addExpr = (AddExpression) currentSelection.getObject();
                    Vector<ArcExpression> vecExpr =  addExpr.getAddExpression();
                    vecExpr.add(new PlaceHolderArcExpression());
                    addExpr = new AddExpression(vecExpr);
                    arcExpression = arcExpression.replace(currentSelection.getObject(), addExpr);
                    updateSelection(addExpr);
                }
            }
        });

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.insets = new Insets(5, 0,0, 0);
        gbc.anchor = GridBagConstraints.WEST;
        arithmeticPanel.add(additionButton, gbc);

        gbc.gridy = 1;
        arithmeticPanel.add(addAdditionPlaceHolderButton, gbc);

        JSeparator separator = new JSeparator(SwingConstants.HORIZONTAL);
        separator.setEnabled(true);
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 2;
        gbc.gridwidth = 3;
        gbc.insets = new Insets(2, 0, 2, 0);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        arithmeticPanel.add(separator,gbc);


        gbc.gridy = 3;
        arithmeticPanel.add(subtractionButton, gbc);

        gbc.gridx = 0;
        gbc.gridy = 4;
        arithmeticPanel.add(scalarJSpinner, gbc);

        gbc.gridx = 1;
        arithmeticPanel.add(scalarButton, gbc);

        gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 1;
        gbc.fill = GridBagConstraints.VERTICAL;
        exprPanel.add(arithmeticPanel, gbc);
    }

    private void initNumberExpressionsPanel() {
        numberExprPanel = new JPanel(new GridBagLayout());
        numberExprPanel.setBorder(BorderFactory.createTitledBorder("Numerical Expressions"));

        Integer current = 1;
        Integer min = 1;
        Integer max = 999;
        Integer step = 1;
        SpinnerNumberModel numberModelNumber = new SpinnerNumberModel(current, min, max, step);
        SpinnerNumberModel numberModelAll = new SpinnerNumberModel(current, min, max, step);

        numberExpressionJSpinner = new JSpinner(numberModelNumber);
        numberExpressionButton = new JButton("Number Expression");



        allExpressionComboBox = new JComboBox();
        allExpressionJSpinner = new JSpinner(numberModelAll);
        allExpressionButton = new JButton("All Expression");

        addColorExpressionButton = new JButton("Edit Color Expression");

        for (ColorType element : context.activeModel().parentNetwork().colorTypes()) {
            allExpressionComboBox.addItem(element);
        }

        numberExpressionJSpinner.setPreferredSize(new Dimension(50, 27));
        numberExpressionJSpinner.setMinimumSize(new Dimension(50, 27));
        numberExpressionJSpinner.setMaximumSize(new Dimension(50, 27));

        allExpressionJSpinner.setPreferredSize(new Dimension(50, 27));
        allExpressionJSpinner.setMinimumSize(new Dimension(50, 27));
        allExpressionJSpinner.setMaximumSize(new Dimension(50, 27));

        allExpressionComboBox.setPreferredSize(new Dimension(150, 27));
        allExpressionComboBox.setMinimumSize(new Dimension(150, 27));
        allExpressionComboBox.setMaximumSize(new Dimension(150, 27));

        numberExpressionButton.setPreferredSize(new Dimension(125, 27));
        numberExpressionButton.setMinimumSize(new Dimension(125, 27));
        numberExpressionButton.setMaximumSize(new Dimension(125, 27));

        allExpressionButton.setPreferredSize(new Dimension(125, 27));
        allExpressionButton.setMinimumSize(new Dimension(125, 27));
        allExpressionButton.setMaximumSize(new Dimension(125, 27));

        addColorExpressionButton.setMaximumSize(new Dimension(125, 27));
        addColorExpressionButton.setMinimumSize(new Dimension(125, 27));
        addColorExpressionButton.setPreferredSize(new Dimension(125, 27));


        allExpressionButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                addAllExpression();
            }
        });

        numberExpressionButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                addNumberExpression();
            }
        });

        addColorExpressionButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                if (currentSelection.getObject() instanceof ColorExpression) {
                    ColorExpression colorExpr = (ColorExpression)currentSelection.getObject();

                    EscapableDialog guiDialog = new EscapableDialog(CreateGui.getApp(), "Edit Color Expression", true);
                    Container contentPane = guiDialog.getContentPane();

                    ColorExpressionDialogPanel cep = new ColorExpressionDialogPanel(guiDialog.getRootPane(), context, colorExpr);
                    contentPane.add(cep);

                    guiDialog.setResizable(true);
                    guiDialog.pack();
                    guiDialog.setLocationRelativeTo(null);
                    guiDialog.setVisible(true);

                    if (cep.clickedOK == true) {
                        ColorExpression expr = cep.getColorExpression();
                        arcExpression = arcExpression.replace(currentSelection.getObject(), expr);
                        updateSelection(expr);
                    }
                }
                else {
                    JOptionPane.showMessageDialog(CreateGui.getApp(), "You have to select a colored placeholder location <+> or a color expression already added in the expression to add a color expression.",
                            "Error", JOptionPane.ERROR_MESSAGE);
                }
            }
        });

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.insets = new Insets(0, 0,5 ,5 );
        numberExprPanel.add(allExpressionJSpinner, gbc);

        gbc.gridx = 1;
        gbc.fill = GridBagConstraints.WEST;
        numberExprPanel.add(allExpressionComboBox, gbc);

        gbc.gridx = 2;
        numberExprPanel.add(allExpressionButton, gbc);

        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.insets = new Insets(0, 0,5 ,5 );
        numberExprPanel.add(numberExpressionJSpinner, gbc);

        gbc.gridx = 2;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        numberExprPanel.add(numberExpressionButton, gbc);

        JSeparator separator = new JSeparator(SwingConstants.HORIZONTAL);
        separator.setEnabled(true);
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 2;
        gbc.gridwidth = 3;
        gbc.insets = new Insets(2, 0, 2, 0);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        numberExprPanel.add(separator,gbc);

        gbc = new GridBagConstraints();
        gbc.gridx = 2;
        gbc.gridy = 3;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        numberExprPanel.add(addColorExpressionButton, gbc);

        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.fill = GridBagConstraints.VERTICAL;
        exprPanel.add(numberExprPanel, gbc);
    }

    private void initColoredTimedGuard() {
        ColorType ct;
        if (!transportArc) {
            ct = ((LocalColoredPlace)objectToBeEditedInput.underlyingTimedInputArc().source()).getColorType();
            colorIntervalPanel = new ColorIntervalAndInvariantEditPanel(context, ct, true, objectToBeEditedInput);
        }
        else {
            ct = ((LocalColoredPlace)objectToBeEditedTransport.underlyingTransportArc().source()).getColorType();
            colorIntervalPanel = new ColorIntervalAndInvariantEditPanel(context, ct, true, objectToBeEditedTransport);
        }

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.insets = new Insets(5, 10, 5, 10);
        gbc.fill = GridBagConstraints.BOTH;
        gbc.anchor = GridBagConstraints.WEST;
        add(colorIntervalPanel, gbc);
    }

    private void initExprField () {
        exprField = new JTextPane();

        StyledDocument doc = exprField.getStyledDocument();

        //Set alignment to be centered for all paragraphs
        MutableAttributeSet standard = new SimpleAttributeSet();
        StyleConstants.setAlignment(standard, StyleConstants.ALIGN_CENTER);
        StyleConstants.setFontSize(standard, 14);
        doc.setParagraphAttributes(0,0, standard,true);

        exprField.setBackground(java.awt.Color.white);

        exprField.setEditable(false);
        exprField.setToolTipText("Tooltip missing");

        exprScrollPane = new JScrollPane(exprField);
        exprScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
        Dimension d = new Dimension(880, 80);
        exprScrollPane.setPreferredSize(d);
        exprScrollPane.setMinimumSize(d);

        exprField.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseReleased(MouseEvent e) {
                if (!exprField.isEditable()) {
                    updateSelection();
                }
            }
        });

        exprField.getDocument().addDocumentListener(new DocumentListener() {
            @Override
            public void insertUpdate(DocumentEvent documentEvent) {
                //TODO: setSaveButtonsEnabled()
            }

            @Override
            public void removeUpdate(DocumentEvent documentEvent) {
                //TODO: setSaveButtonsEnabled()
            }

            @Override
            public void changedUpdate(DocumentEvent documentEvent) {
                //TODO: setSaveButtonsEnabled()
            }
        });

        exprField.addKeyListener(new KeyAdapter() {
            @Override
            public void keyPressed(KeyEvent e) {
                if (!exprField.isEditable()) {
                    //TODO: see line 1232 in CTLQueryDialog for impl example.
                }
            }
        });

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridwidth = 4;
        exprPanel.add(exprScrollPane, gbc);

    }
}
