package pipe.gui.widgets.cpn;


import dk.aau.cs.debug.Logger;
import dk.aau.cs.gui.Context;
import dk.aau.cs.gui.undo.*;
import dk.aau.cs.model.tapn.*;
import dk.aau.cs.model.tapn.Colored.*;
import dk.aau.cs.model.tapn.Colored.Color;
import dk.aau.cs.util.RequireException;
import pipe.gui.CreateGui;
import pipe.gui.Pipe;
import pipe.gui.graphicElements.cpn.ColoredPlaceComponent;
import pipe.gui.widgets.EscapableDialog;
import pipe.gui.widgets.PlaceEditorPanel;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;
import java.util.List;

public class ColoredPlaceEditorPanel extends PlaceEditorPanel {
    private static final long serialversionUID = 1L;
    private JRootPane rootPane;

    private JPanel tokenPanel;
    private JPanel colorTypePanel;
    private JLabel colortypeLabel;
    private JComboBox<ColorType> colorTypeComboBox;
    private DefaultListModel<ColoredToken> listModel;
    private JList tokenList;
    private JButton addElementButton;
    private JButton removeSelectedElementButton;


    private JPanel timeInvariantColorPanel;

    private List<ColorType> colorTypes;
    private ColorType colorType;
    private List<TimedToken> ct;
    private boolean makeNewShared = false;

    private ColorComboboxPanel tokenColorComboboxPanel;
    private ColorIntervalAndInvariantEditPanel invariantEditPanel;

    private ColoredPlaceComponent place;


    public ColoredPlaceEditorPanel(JRootPane rootPane, ColoredPlaceComponent placeComponent, Context context) {
        super(rootPane, placeComponent, context);
        this.rootPane = rootPane;
        this.place = placeComponent;
        initColoredComponents();
    }

    public ColoredPlaceEditorPanel(JRootPane rootPane, ColoredPlaceComponent placeComponent, ColorType colorType , Context context) {
        super(rootPane, placeComponent, context);
        this.colorType = colorType;
        this.rootPane = rootPane;
        this.place = placeComponent;
        initColoredComponents();
    }

    private void initColoredComponents() {
        colorTypes = new ArrayList();

        initColorInvariantPanel();
        initColorTypePanel();
        initTokensPanel();
        initEvents();
        setInitialComboBoxValue();
        writeTokensToList();
        markingSpinner.setVisible(false);
        markingLabel.setVisible(false);
        setColoredTimeInvariants();
    }

    private void setColoredTimeInvariants() {

    }

    private void initColorInvariantPanel() {
        timeInvariantColorPanel = new JPanel(new GridBagLayout());
        timeInvariantColorPanel.setBorder(BorderFactory.createTitledBorder("Time invariant for colors"));
      invariantEditPanel = new ColorIntervalAndInvariantEditPanel(context, colorType, false, place);

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        timeInvariantColorPanel.add(invariantEditPanel, gbc);

        gbc.gridx = 0;
        gbc.gridy = 2;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(3, 3, 3, 3);
        add(timeInvariantColorPanel, gbc);
    }

    private void writeTokensToList() {
        listModel.clear();
        ct = context.network().marking().getTokensFor(place.underlyingPlace());
        List<Color> ctList = new ArrayList();
        for (TimedToken element : ct) {
            ColoredToken token = (ColoredToken)element;
            listModel.addElement(token);
        }
    }


    private void setInitialComboBoxValue() {
        if (colorType != null) {
            colorTypeComboBox.setSelectedIndex(colorTypes.indexOf(colorType));
        }
        else if (colorTypes.size() != 0) {
            colorTypeComboBox.setSelectedIndex(0);
        }
    }

    private void setNewColorType(ColorType colorType) {
        listModel.clear();
        this.colorType = colorType;
        remove(tokenPanel);
        remove(timeInvariantColorPanel);
        initTokensPanel();
        initColorInvariantPanel();
        revalidate();
    }


    private void initEvents() {
        makeSharedButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                makeNewShared = true;
                if (doOK()) {
                    makeShared();
                }
                makeNewShared = false;
            }
        });

        okButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                if (doOKColored() && doOK()) {
                    exit();
               }
            }
        });

    }

    private void onRemove() {
        listModel.removeElementAt(tokenList.getSelectedIndex());
    }

    private void onAdd() {
        ColoredToken ct;
        if (colorType instanceof ProductType) {
            Vector<Color> colorVector = new Vector();
            for (int i = 0; i < tokenColorComboboxPanel.getColorTypeComboBoxesArray().length ; i++) {
                colorVector.add((Color)tokenColorComboboxPanel.getColorTypeComboBoxesArray()[i].getItemAt(tokenColorComboboxPanel.getColorTypeComboBoxesArray()[i].getSelectedIndex()));
            }
            Color tempColor = new Color(colorType, 0, colorVector);
            ct = new ColoredToken(place.underlyingPlace(), tempColor);
            listModel.addElement(ct);
        }
        else {
            ct = new ColoredToken(place.underlyingPlace(), (Color)tokenColorComboboxPanel.getColorTypeComboBoxesArray()[0].getItemAt(tokenColorComboboxPanel.getColorTypeComboBoxesArray()[0].getSelectedIndex()));
            listModel.addElement(ct);
        }
    }

    public boolean doOKColored() {
        ArrayList<TimedToken> tokenList = new ArrayList(context.activeModel().marking().getTokensFor(place.underlyingPlace()));

        for (TimedToken token : tokenList) {
            context.activeModel().marking().remove(token);
        }

        for (int i = 0; i < listModel.size(); i++) {
            context.activeModel().marking().add(listModel.get(i));
        }
        List<ColoredTimeInvariant> ctiList = new ArrayList<ColoredTimeInvariant>();
        for (int i = 0; i < invariantEditPanel.getTimeConstraintModel().size(); i++) {
            ctiList.add((ColoredTimeInvariant) invariantEditPanel.getTimeConstraintModel().get(i));
        }
        if(place.underlyingPlace() instanceof LocalColoredPlace) {
            ((LocalColoredPlace) place.underlyingPlace()).setCtiList(ctiList);
            ((LocalColoredPlace) place.underlyingPlace()).setColorType(colorType);
        }
        else if(place.underlyingPlace() instanceof SharedColoredPlace) {
            ((SharedColoredPlace) place.underlyingPlace()).setCtiList(ctiList);
            ((SharedColoredPlace) place.underlyingPlace()).setColorType(colorType);
        }
        return true;
    }

    @Override //Because we don't choose the number of markings with the Spinner in a colored place we have to rewrite the doOK function to ignore spinner value and look at number of tokens (list elements) instead.
    protected boolean doOK() {
        if (!isUrgencyOK() && !invariantInf.isSelected()) {
            return false;
        }
        int newTokens = listModel.getSize();
        if (newTokens > Pipe.MAX_NUMBER_OF_TOKENS_ALLOWED) {
            JOptionPane.showMessageDialog(this,"It is allowed to have at most " + Pipe.MAX_NUMBER_OF_TOKENS_ALLOWED + " tokens in a place.", "Error", JOptionPane.ERROR_MESSAGE);
            return false;
        }
        context.undoManager().newEdit(); // new "transaction"

        TimedPlace underlyingPlace = place.underlyingPlace();

        SharedPlace selectedPlace = (SharedPlace)sharedPlacesComboBox.getSelectedItem();
        if (sharedCheckBox.isSelected() && !selectedPlace.equals(underlyingPlace)) {
            Command command = new MakePlaceSharedCommand(context.activeModel(), selectedPlace, place.underlyingPlace(), place, context.tabContent());
            context.undoManager().addEdit(command);
            try {
                command.redo();
            } catch (RequireException e ) {
                context.undoManager().undo();
                JOptionPane.showMessageDialog(this, "Another place in the same component is already shared under that name", "Error", JOptionPane.ERROR_MESSAGE);
                return false;
            }

        }
        else if(!sharedCheckBox.isSelected()) {
            if (underlyingPlace.isShared()) {
                String uniqueName = context.nameGenerator().getNewPlaceName(context.activeModel());
                Command unShareCmd = new UnsharePlaceCommand(context.activeModel(), (SharedPlace)underlyingPlace, new LocalColoredPlace(uniqueName), place);
                unShareCmd.redo();
                context.undoManager().addEdit(unShareCmd);
            }
        }

        String newName = nameTextField.getText();
        String oldName = place.underlyingPlace().name();
        if (context.activeModel().isNameUsed(newName) && !oldName.equalsIgnoreCase(newName)) {
            context.undoManager().undo();
            JOptionPane.showMessageDialog(this, "The specified name is already used by another place or transition", "Error", JOptionPane.ERROR_MESSAGE);
            return false;
        }

        if(place.underlyingPlace() instanceof LocalColoredPlace) {
            Command renameCommand =  renameCommand = new RenameTimedPlaceCommand(context.tabContent(), (LocalColoredPlace) place.underlyingPlace(), oldName, newName);
            context.undoManager().addEdit(renameCommand);
            try {
                renameCommand.redo();
            } catch (RequireException e) {
                context.undoManager().undo();
                JOptionPane.showMessageDialog(this, "Acceptable names for transitions are defined by the regular expression:\n[a-zA-Z][_a-zA-Z0-9]*\n\nNote that \"true\" and \"false\" are reserved keywords.", "Error", JOptionPane.ERROR_MESSAGE);
                return false;
            }
            context.nameGenerator().updateIndices(context.activeModel(), newName);
        }


        if (makeNewShared) {
           Command command = new MakePlaceNewSharedCommand(context.activeModel(), newName, place.underlyingPlace(), place, context.tabContent());
           context.undoManager().addEdit(command);
           try {
               command.redo();
           } catch (RequireException e) {
               context.undoManager().undo();
               JOptionPane.showMessageDialog(this, "A transition or place with the specified name already exists, or the specified name is invalid.\n\nAcceptable names for transitions are defined by the regular expression:\n[a-zA-Z][_a-zA-Z0-9]*\n\nNote that \"true\" and \"false\" are reserved keywords.", "Error", JOptionPane.ERROR_MESSAGE);
               	return false;
           }
        }

        if (newTokens != place.underlyingPlace().numberOfTokens())   {
            Command command = new TimedPlaceMarkingEdit(place, newTokens - place.underlyingPlace().numberOfTokens());
            command.redo();
            context.undoManager().addEdit(command);
        }
    /*
        ColoredTimeInvariant newInvariant = constructColoredInvariant();
        ColoredTimeInvariant oldInvariant = (ColoredTimeInvariant)place.underlyingPlace().invariant();
        if (!newInvariant.equals(oldInvariant)) {
            context.undoManager().addEdit(new ChangedInvariantCommand(place.underlyingPlace(), oldInvariant, newInvariant));
            place.setInvariant(newInvariant);
        }
        */
        if ((place.getAttributesVisible() && !attributesCheckBox.isSelected()) || (!place.getAttributesVisible() && attributesCheckBox.isSelected())) {
            place.toggleAttributesVisible();
        }

        if(place.underlyingPlace() instanceof LocalColoredPlace) {
            if (((LocalColoredPlace) place.underlyingPlace()).getColorType() == null) {
                ((LocalColoredPlace) place.underlyingPlace()).setColorType(colorType);
            } else if (!colorType.getName().equals(((LocalColoredPlace) place.underlyingPlace()).getColorType().getName())) {
                ((LocalColoredPlace) place.underlyingPlace()).setColorType(colorType);
            }
        }else if (((SharedColoredPlace)place.underlyingPlace()).getColorType() == null) {
            ((SharedColoredPlace) place.underlyingPlace()).setColorType(colorType);
        }

        place.update(true);
        place.repaint();
        context.network().buildConstraints();

        return true;
    }

    private void initColorTypePanel() {
        colorTypePanel = new JPanel();
        colorTypePanel.setLayout(new GridBagLayout());
        colorTypePanel.setBorder(new TitledBorder("Color Type"));

        colortypeLabel = new JLabel();
        colortypeLabel.setText("Color Type:");

        colorTypeComboBox = new JComboBox();
        colorTypes = context.network().colorTypes();

        for (ColorType element : colorTypes) {
            colorTypeComboBox.addItem(element);
        }

        colorTypeComboBox.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                if (!listModel.isEmpty() && !invariantEditPanel.getTimeConstraintModel().isEmpty()) {
                    int dialogResult = JOptionPane.showConfirmDialog(null, "Are you sure you want to change the color type for this place?\n" +
                            "All tokens and time invariants for colors will be deleted.","alert", JOptionPane.YES_NO_OPTION);
                    if (dialogResult == JOptionPane.YES_OPTION) {
                        setNewColorType(colorTypeComboBox.getItemAt(colorTypeComboBox.getSelectedIndex()));
                    }
                    else { // NO.OPTION - we set the color type to the previous selected one
                        for (int i = 0; i <  colorTypeComboBox.getItemCount(); i++) {
                            if (colorType.getName().equals(colorTypeComboBox.getItemAt(i).getName())) {
                                colorTypeComboBox.setSelectedIndex(i);
                            }
                        }
                    }
                } else {
                    setNewColorType(colorTypeComboBox.getItemAt(colorTypeComboBox.getSelectedIndex()));
                }
            }
        });

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.insets = new Insets(3, 3, 3, 3);
        colorTypePanel.add(colortypeLabel, gbc);

        Dimension colorTypeComboBoxSize = new Dimension(500, 30);
        colorTypeComboBox.setPreferredSize(colorTypeComboBoxSize);
        colorTypeComboBox.setMinimumSize(colorTypeComboBoxSize);
        colorTypeComboBox.setPreferredSize(colorTypeComboBoxSize);

        gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.insets = new Insets(3, 3, 3,3 );
        colorTypePanel.add(colorTypeComboBox, gbc);

        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(3, 3, 3, 3);
        add(colorTypePanel, gbc);
    }

    private void initTokensPanel() {
        tokenPanel = new JPanel();
        tokenPanel.setLayout(new GridBagLayout());
        tokenPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Tokens"));

        tokenColorComboboxPanel = new ColorComboboxPanel(colorType, "colors");
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.SOUTH;
        tokenPanel.add(tokenColorComboboxPanel, gbc);
        //Logger.log(tokenColorComboboxPanel.getColorTypeComboBoxesArray()[0].getItemAt(0).toString());


        listModel = new DefaultListModel();
        tokenList = new JList(listModel);
        JScrollPane tokenListScrollPane = new JScrollPane(tokenList);
        tokenListScrollPane.setViewportView(tokenList);
        tokenListScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
        tokenListScrollPane.setPreferredSize(new Dimension(600, 200));
        tokenListScrollPane.setMaximumSize(new Dimension(400, 200));
        tokenListScrollPane.setMinimumSize(new Dimension(400, 200));
        tokenListScrollPane.setBorder(BorderFactory.createTitledBorder( "Tokens"));




        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.gridwidth = 3;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(3, 3, 3,3);
        //tokenPanel.add(tokenListScrollPane, gbc);
        tokenPanel.add(tokenListScrollPane, gbc);


        addElementButton = new JButton("Add");
        Dimension buttonSize = new Dimension(100, 27);
        addElementButton.setPreferredSize(buttonSize);
        addElementButton.setMinimumSize(buttonSize);
        addElementButton.setMaximumSize(buttonSize);

        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.SOUTHWEST;
        gbc.insets = new Insets(3, 3, 3,3);
        tokenPanel.add(addElementButton, gbc);

        addElementButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                onAdd();
            }
        });

        removeSelectedElementButton = new JButton("Remove");


        removeSelectedElementButton.setPreferredSize(buttonSize);
        removeSelectedElementButton.setMinimumSize(buttonSize);
        removeSelectedElementButton.setMaximumSize(buttonSize);

        removeSelectedElementButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                onRemove();
            }
        });

        gbc.gridx = 2;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.SOUTHWEST;
        gbc.insets = new Insets(3, 3, 3, 3);
        tokenPanel.add(removeSelectedElementButton, gbc);

        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 4;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(3, 3, 3, 3);
        add(tokenPanel, gbc);
    }

}
