package pipe.gui.widgets.cpn;

import dk.aau.cs.model.tapn.Colored.ColorType;
import dk.aau.cs.model.tapn.Colored.Variable;
import dk.aau.cs.model.tapn.TimedArcPetriNetNetwork;
import pipe.gui.CreateGui;
import pipe.gui.widgets.ConstantsPane;
import pipe.gui.widgets.EscapableDialog;
import pipe.gui.widgets.RequestFocusListener;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

public class VariablesDialogPanel extends javax.swing.JPanel {
    private static final long serialVersionUID = 1L;
    private JRootPane rootPane;
    private TimedArcPetriNetNetwork network;
    private EscapableDialog dialog;
    private Variable variable;
    private List<ColorType> colorTypes;
    private ConstantsPane.VariablesListModel listModel;

    private String oldName;

    JTextField nameTextField;
    Dimension size;
    JLabel nameLabel;

    JComboBox colorTypeComboBox;
    JLabel colorTypeLabel;

    JPanel container;
    JPanel buttonContainer;
    JButton okButton;
    JButton cancelButton;

    public VariablesDialogPanel() throws IOException {
        initComponents();
    }

    public VariablesDialogPanel(JRootPane pane, ConstantsPane.VariablesListModel listModel, TimedArcPetriNetNetwork network) throws IOException {
        rootPane = pane;
        oldName = "";
        this.network = network;
        this.listModel = listModel;
        initComponents();
        nameTextField.setText(oldName);


    }

    public VariablesDialogPanel(JRootPane pane, ConstantsPane.VariablesListModel listModel, TimedArcPetriNetNetwork network, Variable variable) throws IOException {
        rootPane = pane;
        this.variable = variable;
        // set combobox value to the already chosen ColorType
        oldName = variable.getName();
        this.network = network;
        this.listModel = listModel;
        initComponents();
        nameTextField.setText(oldName);
    }

    public void showDialog() {
        dialog = new EscapableDialog(CreateGui.getApp(),
                "Edit Variable", true);
        dialog.add(container);
        dialog.getRootPane().setDefaultButton(okButton);
        dialog.setResizable(false);
        dialog.pack();
        dialog.setLocationRelativeTo(null);
        dialog.setVisible(true);
    }

    private void initComponents() throws IOException {
        container = new JPanel();
        buttonContainer = new JPanel();
        container.setLayout(new GridBagLayout());
        size = new Dimension(330, 30);

        createCancelButton();
        createColorTypeLabel();
        createcolorTypesComboBox();
        createNameLabel();
        createNameTextField();
        createOKButton();

        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                onOK();
            }
        });

        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                exit();
            }
        });

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets = new Insets(0, 8, 5, 8);
        gbc.gridx = 1;
        gbc.gridy = 2;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.EAST;
        container.add(buttonContainer,gbc);
    }

    private void createNameTextField() {
        nameTextField = new javax.swing.JTextField();
        nameTextField.setPreferredSize(size);
        nameTextField.addAncestorListener(new RequestFocusListener());
        nameTextField.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                okButton.requestFocusInWindow();
                okButton.doClick();
            }
        });

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(4, 4, 2, 4);
        container.add(nameTextField,gbc);
    }

    private void createNameLabel() {
        nameLabel = new JLabel();
        nameLabel.setText("Name: ");
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = 1;
        gbc.insets = new Insets(4, 4, 2, 4);
        gbc.anchor = GridBagConstraints.WEST;
        container.add(nameLabel,gbc);
    }

    private void createColorTypeLabel() {
        colorTypeLabel = new javax.swing.JLabel();
        colorTypeLabel.setText("Color type: ");
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets = new Insets(2, 4, 2, 4);
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.WEST;
        container.add(colorTypeLabel,gbc);
    }


    private void createcolorTypesComboBox() {
        colorTypes = new ArrayList<ColorType>();
        colorTypes = network.colorTypes();

        colorTypeComboBox = new javax.swing.JComboBox();

        int variableIndex = 0;
        for (ColorType element : colorTypes) {
            colorTypeComboBox.addItem(element.getName());
            if (variable != null) {
                if (element.getName().equals(variable.getColorType().getName())) {
                    variableIndex = colorTypeComboBox.getItemCount() - 1;
                }
            }

        }
        colorTypeComboBox.setSelectedIndex(variableIndex);

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets = new Insets(2, 4, 2, 4);
        gbc.gridx = 1;
        gbc.gridy = 1;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        container.add(colorTypeComboBox,gbc);
    }

    private void createOKButton() {
        okButton = new JButton();
        okButton.setText("OK");
        okButton.setMaximumSize(new java.awt.Dimension(100, 25));
        okButton.setMinimumSize(new java.awt.Dimension(100, 25));
        okButton.setPreferredSize(new java.awt.Dimension(100, 25));
        okButton.setMnemonic(KeyEvent.VK_O);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.anchor = java.awt.GridBagConstraints.WEST;
        gbc.insets = new java.awt.Insets(5, 5, 5, 5);
        buttonContainer.add(okButton,gbc);
    }

    private void createCancelButton() {
        cancelButton = new JButton();
        cancelButton.setText("Cancel");
        cancelButton.setMaximumSize(new java.awt.Dimension(100, 25));
        cancelButton.setMinimumSize(new java.awt.Dimension(100, 25));
        cancelButton.setPreferredSize(new java.awt.Dimension(100, 25));
        cancelButton.setMnemonic(KeyEvent.VK_C);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.gridwidth = java.awt.GridBagConstraints.RELATIVE;
        gbc.anchor = GridBagConstraints.EAST;
        buttonContainer.add(cancelButton,gbc);
    }

    private void exit() {
        dialog.setVisible(false);
    }

    private void onOK() {
        if (colorTypeComboBox.getSelectedItem() == null) {
            JOptionPane.showMessageDialog(
                    CreateGui.getApp(),
                    "You have to choose a color type for the variable. If none are present you have to create one first under variables.",
                    "Error", JOptionPane.ERROR_MESSAGE);
            colorTypeComboBox.requestFocusInWindow();
            return;
        }
        String newName = nameTextField.getText();
        if (newName.trim().isEmpty()) {
            JOptionPane.showMessageDialog(
                    CreateGui.getApp(),
                    "You have to enter a name in the textfield in order to create a new variable",
                    "Error", JOptionPane.ERROR_MESSAGE);
            colorTypeComboBox.requestFocusInWindow();
            return;
        }
        if (!Pattern.matches("[a-zA-Z]([\\_a-zA-Z0-9])*", newName)) {
            JOptionPane
                    .showMessageDialog(
                            CreateGui.getApp(),
                            "Acceptable names for variables are defined by the regular expression:\n[a-zA-Z][_a-zA-Z0-9]*",
                            "Error", JOptionPane.ERROR_MESSAGE);
            nameTextField.requestFocusInWindow();
            return;
        }
        if (network.isNameUsedForVariable(newName) && variable.getColorType().equals(colorTypeComboBox.getSelectedIndex())) {
            JOptionPane
                    .showMessageDialog(
                            CreateGui.getApp(),
                            "There is already another variable with the same name.\n\n"
                                    + "Choose a different name for the variable.",
                            "Error", JOptionPane.ERROR_MESSAGE);
            nameTextField.requestFocusInWindow();
            return;
        }
        if (!oldName.equals("") && !oldName.equalsIgnoreCase(newName) && network.isNameUsedForVariable(newName) && variable.getColorType().equals(colorTypeComboBox.getSelectedIndex())) {
            JOptionPane
                    .showMessageDialog(
                            CreateGui.getApp(),
                            "There is already another variable with the same name.\n\n"
                                    + "Choose a different name for the variable.",
                            "Error", JOptionPane.ERROR_MESSAGE);
            nameTextField.requestFocusInWindow();
            return;
        }
        if (!oldName.equals("")) {
            //TODO MAKE SURE THAT ID DOES NOT DESTROY ANYTHING
            network.updateVariable(oldName, new Variable(nameTextField.getText(),"Var" + nameTextField.getText(), colorTypes.get(colorTypeComboBox.getSelectedIndex())));
            listModel.updateName();
        }
        else {
            //TODO SAME AS THE ONE OVER
            listModel.addElement(new Variable(nameTextField.getText(),"Var" + nameTextField.getText(), colorTypes.get(colorTypeComboBox.getSelectedIndex())));
        }
        exit();

    }

}
